import phanterpwa.frontend.components.widgets as widgets
import phanterpwa.frontend.helpers as helpers
import plugins.codemirror as codemirror
from org.transcrypt.stubs.browser import __pragma__

__pragma__('alias', "jQuery", "$")
__pragma__('skip')
jQuery = sessionStorage = JSON = M = js_undefined = window =\
    __new__ = FormData = console = localStorage = document = this = 0
__pragma__('noskip')


DIV = helpers.XmlConstructor.tagger("div")
LABEL = helpers.XmlConstructor.tagger("label")
I = helpers.XmlConstructor.tagger("i")
H2 = helpers.XmlConstructor.tagger("h2")
H3 = helpers.XmlConstructor.tagger("h3")
FORM = helpers.XmlConstructor.tagger("form")
SPAN = helpers.XmlConstructor.tagger("span")
IMG = helpers.XmlConstructor.tagger("img", True)
HR = helpers.XmlConstructor.tagger("hr", True)
UL = helpers.XmlConstructor.tagger("ul")
STRONG = helpers.XmlConstructor.tagger("strong")
LI = helpers.XmlConstructor.tagger("li")
INPUT = helpers.XmlConstructor.tagger("input", True)
BR = helpers.XmlConstructor.tagger("br", True)
P = helpers.XmlConstructor.tagger("p")
A = helpers.XmlConstructor.tagger("a")
I18N = helpers.I18N
CONCATENATE = helpers.CONCATENATE
XML = helpers.XML
XSECTION = helpers.XSECTION


codemirrorselectwearandroidnolabel = """
# your code

your_instance = widgets.Select(
    "selectwearandroidnolabel",
    placeholder="Placeholder Widget",
    data_set={
        "banana": "Banana",
        "abacate": "Abacate",
        "doce": "Doce"
    }
),

# your code
"""


codemirrorselectwearshadowsnolabel = """
# your code

your_instance = widgets.Select(
    "selectwearshadowsnolabel",
    placeholder="Placeholder Widget",
    data_set={
        "banana": "Banana",
        "abacate": "Abacate",
        "doce": "Doce"
    },
    wear="shadows"
),

# your code
"""


codemirrorselectwearelegantnolabel = """
# your code

your_instance = widgets.Select(
    "selectwearelegantnolabel",
    placeholder="Placeholder Widget",
    data_set={
        "banana": "Banana",
        "abacate": "Abacate",
        "doce": "Doce"
    },
    wear="elegant"
),

# your code
"""


codemirrorselectwearandroid = """
# your code

your_instance = widgets.Select(
    "selectwearandroid",
    label="Label Widget",
    placeholder="Placeholder Widget",
    data_set={
        "banana": "Banana",
        "abacate": "Abacate",
        "doce": "Doce"
    }
),

# your code
"""


codemirrorselectwearshadows = """
# your code

your_instance = widgets.Select(
    "selectwearshadows",
    label="Label Widget",
    placeholder="Placeholder Widget",
    data_set={
        "banana": "Banana",
        "abacate": "Abacate",
        "doce": "Doce"
    },
    wear="shadows"
),

# your code
"""


codemirrorselectwearelegant = """
# your code

your_instance = widgets.Select(
    "selectwearelegant",
    label="Label Widget",
    placeholder="Placeholder Widget",
    data_set={
        "banana": "Banana",
        "abacate": "Abacate",
        "doce": "Doce"
    },
    wear="elegant"
),

# your code
"""


codemirrorselectwearandroidicon = """
# your code

your_instance = widgets.Select(
    "selectwearandroidicon",
    label="Label Widget",
    placeholder="Placeholder Widget",
    icon=I(_class="fab fa-telegram-plane"),
    data_set={
        "banana": "Banana",
        "abacate": "Abacate",
        "doce": "Doce"
    }
),

# your code
"""


codemirrorselectwearshadowsicon = """
# your code

your_instance = widgets.Select(
    "selectwearshadowsicon",
    label="Label Widget",
    placeholder="Placeholder Widget",
    icon=I(_class="fab fa-telegram-plane"), 
    data_set={
        "banana": "Banana",
        "abacate": "Abacate",
        "doce": "Doce"
    },
    wear="shadows"
),

# your code
"""


codemirrorselectweareleganticon = """
# your code

your_instance = widgets.Select(
    "selectweareleganticon",
    label="Label Widget",
    placeholder="Placeholder Widget",
    icon=I(_class="fab fa-telegram-plane"), 
    data_set={
        "banana": "Banana",
        "abacate": "Abacate",
        "doce": "Doce"
    },
    wear="elegant"
),

# your code
"""


codemirrorselectwearandroidiconnolabel = """
# your code

your_instance = widgets.Select(
    "selectwearandroidiconnolabel",
    placeholder="Placeholder Widget",
    icon=I(_class="fab fa-telegram-plane"),
    data_set={
        "banana": "Banana",
        "abacate": "Abacate",
        "doce": "Doce"
    }
),

# your code
"""


codemirrorselectwearshadowsiconnolabel = """
# your code

your_instance = widgets.Select(
    "selectwearshadowsiconnolabel",
    placeholder="Placeholder Widget",
    icon=I(_class="fab fa-telegram-plane"),
    data_set={
        "banana": "Banana",
        "abacate": "Abacate",
        "doce": "Doce"
    },
    wear="shadows"
),

# your code
"""


codemirrorselectweareleganticonnolabel = """
# your code

your_instance = widgets.Select(
    "selectweareleganticonnolabel",
    placeholder="Placeholder Widget",
    icon=I(_class="fab fa-telegram-plane"),
    data_set={
        "banana": "Banana",
        "abacate": "Abacate",
        "doce": "Doce"
    },
    wear="elegant"
),

# your code
"""


codemirrorsimpleselect = """
# your code

your_instance = widgets.Select(
    "simpleselect",
    label="Label Widget",
    data_set={
        "banana": "banana",
        "abacate": "abacate",
        "doce": "doce"
    }
)

# your code
"""


codemirrorselectcanempty = """
# your code

your_instance = widgets.Select(
    "selectcanempty",
    label="Label Widget",
    can_empty=True,
    data_set={
        "banana": "banana",
        "abacate": "abacate",
        "doce": "doce"
    }
)

# your code
"""


codemirrorselectcanempty = """
# your code

your_instance = widgets.Select(
    "selectcanempty",
    label="Label Widget",
    can_empty=True,
    data_set={
        "banana": "banana",
        "abacate": "abacate",
        "doce": "doce"
    }
)

# your code
"""


codemirrorselectcanemptyplaceholder = """
# your code

your_instance = widgets.Select(
    "selectcanemptyplaceholder",
    label="Label Widget",
    placeholder="Placeholder Widget",
    can_empty=True,
    data_set={
        "banana": "banana",
        "abacate": "abacate",
        "doce": "doce"
    }
)

# your code
"""


codemirrorselectcanemptyeditableplaceholder = """
# your code

your_instance = widgets.Select(
    "selectcanemptyeditableplaceholder",
    label="Label Widget",
    placeholder="Placeholder Widget",
    editable=True,
    can_empty=True,
    data_set={
        "banana": "banana",
        "abacate": "abacate",
        "doce": "doce"
    }
)

# your code
"""


codemirrorselectcanemptyeditable = """
# your code

your_instance = widgets.Select(
    "selectcanemptyeditable",
    label="Label Widget",
    editable=True,
    can_empty=True,
    data_set={
        "banana": "banana",
        "abacate": "abacate",
        "doce": "doce"
    }
)

# your code
"""

multselect1_source = """
# your code

your_instance = widgets.MultSelect(
    "the_multiselect1",
    label="Label Widget",
    placeholder="Placeholder Widget",
    data_set={
        "banana": "Banana",
        "abacate": "Abacate",
        "doce": "Doce"
    },
)

# your code

"""
multselect2_source = """
# your code

your_instance = widgets.MultSelect(
    "the_multiselect2",
    label="Label Widget",
    placeholder="Placeholder Widget",
    editable=True,
    data_set={
        "banana": "Banana",
        "abacate": "Abacate",
        "doce": "Doce"
    },
)

# your code

"""

cod_mult_sec_styles = """
import phanterpwa.frontend.components.widgets as widgets
import phanterpwa.frontend.helpers as helpers

CONCATENATE = helpers.CONCATENATE
DIV = helpers.XmlConstructor.tagger("div")
H3 = helpers.XmlConstructor.tagger("h3")

your_instance = CONTATENATE(
    H3("Style android"),
    DIV(
        widgets.MultSelect(
            "the_multiselect1stylesnolabelnoplaceholder",
            data_set={
                "banana": "Banana",
                "abacate": "Abacate",
                "doce": "Doce"
            },
        ),
        widgets.MultSelect(
            "the_multiselect1stylesnolabelnoplaceh",
            placeholder="Placeholder Widget",
            data_set={
                "banana": "Banana",
                "abacate": "Abacate",
                "doce": "Doce"
            },
        ),
        widgets.MultSelect(
            "the_multiselect1styles",
            label="Label Widget",
            placeholder="Placeholder Widget",
            data_set={
                "banana": "Banana",
                "abacate": "Abacate",
                "doce": "Doce"
            },
        ),
        widgets.MultSelect(
            "the_multiselect1stylesnolabelicon",
            placeholder="Placeholder Widget",
            data_set={
                "banana": "Banana",
                "abacate": "Abacate",
                "doce": "Doce"
            },
            icon=I(_class="fab fa-telegram-plane")
        ),
        widgets.MultSelect(
            "the_multiselect1stylesicon",
            label="Label Widget",
            placeholder="Placeholder Widget",
            data_set={
                "banana": "Banana",
                "abacate": "Abacate",
                "doce": "Doce"
            },
            icon=I(_class="fab fa-telegram-plane")
        ),
        _class="widget_input_example"
    ),
    H3("Style shadows"),
    DIV(
        widgets.MultSelect(
            "the_multiselect1stylesshadowsnolabelnoplaceholder",
            wear="shadows",
            data_set={
                "banana": "Banana",
                "abacate": "Abacate",
                "doce": "Doce"
            },
        ),
        widgets.MultSelect(
            "the_multiselect1stylesshadowsnolabelnoplaceh",
            placeholder="Placeholder Widget",
            wear="shadows",
            data_set={
                "banana": "Banana",
                "abacate": "Abacate",
                "doce": "Doce"
            },
        ),
        widgets.MultSelect(
            "the_multiselect1stylesshadows",
            label="Label Widget",
            placeholder="Placeholder Widget",
            wear="shadows",
            data_set={
                "banana": "Banana",
                "abacate": "Abacate",
                "doce": "Doce"
            },
        ),
        widgets.MultSelect(
            "the_multiselect1stylesshadowsnolabelicon",
            placeholder="Placeholder Widget",
            wear="shadows",
            data_set={
                "banana": "Banana",
                "abacate": "Abacate",
                "doce": "Doce"
            },
            icon=I(_class="fab fa-telegram-plane")
        ),
        widgets.MultSelect(
            "the_multiselect1stylesshadowsicon",
            label="Label Widget",
            placeholder="Placeholder Widget",
            wear="shadows",
            data_set={
                "banana": "Banana",
                "abacate": "Abacate",
                "doce": "Doce"
            },
            icon=I(_class="fab fa-telegram-plane")
        ),
        _class="widget_input_example"
    ),
    H3("Style elegant"),
    DIV(
        widgets.MultSelect(
            "the_multiselect1elegantnolabelnoplaceholder",
            wear="elegant",
            data_set={
                "banana": "Banana",
                "abacate": "Abacate",
                "doce": "Doce"
            },
        ),
        widgets.MultSelect(
            "the_multiselect1elegantnolabelnoplaceh",
            placeholder="Placeholder Widget",
            wear="elegant",
            data_set={
                "banana": "Banana",
                "abacate": "Abacate",
                "doce": "Doce"
            },
        ),
        widgets.MultSelect(
            "the_multiselect1elegant",
            label="Label Widget",
            placeholder="Placeholder Widget",
            wear="elegant",
            data_set={
                "banana": "Banana",
                "abacate": "Abacate",
                "doce": "Doce"
            },
        ),
        widgets.MultSelect(
            "the_multiselect1elegantnolabelicon",
            placeholder="Placeholder Widget",
            wear="elegant",
            data_set={
                "banana": "Banana",
                "abacate": "Abacate",
                "doce": "Doce"
            },
            icon=I(_class="fab fa-telegram-plane")
        ),
        widgets.MultSelect(
            "the_multiselect1eleganticon",
            label="Label Widget",
            placeholder="Placeholder Widget",
            wear="elegant",
            data_set={
                "banana": "Banana",
                "abacate": "Abacate",
                "doce": "Doce"
            },
            icon=I(_class="fab fa-telegram-plane")
        ),
        _class="widget_input_example"
    ),
)
"""


class Selects():
    def __init__(self, gatehandler):
        self.gatehandler = gatehandler
        gatehandler.Selects = self
        html = DIV(
            DIV(
                H2(I18N("Select - Styles (Wear)")),
                HR(),
                XSECTION(
                    LABEL(I18N("Examples"), ' 01 (wear, label, icon)'),
                    DIV(
                        STRONG(I18N("Code"), ":"),
                        codemirror.CodeMirrorHelper(
                            "codemirrorselectwearandroidnolabel",
                            **{
                                "value": codemirrorselectwearandroidnolabel,
                                "mode": "python",
                                "lineNumbers": True
                            }
                        ),
                        STRONG(I18N("Results"), ":"),
                        DIV(
                            widgets.Select(
                                "selectwearandroidnolabel",
                                placeholder="Placeholder Widget",
                                data_set={
                                    "banana": "Banana",
                                    "abacate": "Abacate",
                                    "doce": "Doce"
                                }
                            ),
                            _class="widget_input_example"
                        ),
                        BR(),
                        H3("Label example"),
                        STRONG(I18N("Code"), ":"),
                        codemirror.CodeMirrorHelper(
                            "codemirrorselectwearandroid",
                            **{
                                "value": codemirrorselectwearandroid,
                                "mode": "python",
                                "lineNumbers": True
                            }
                        ),
                        STRONG(I18N("Results"), ":"),
                        DIV(
                            widgets.Select(
                                "selectwearandroid",
                                label="Label Widget",
                                placeholder="Placeholder Widget",
                                data_set={
                                    "banana": "Banana",
                                    "abacate": "Abacate",
                                    "doce": "Doce"
                                }
                            ),
                            _class="widget_input_example"
                        ),
                        BR(),
                        H3("Icon examples"),
                        STRONG(I18N("Code"), ":"),
                        codemirror.CodeMirrorHelper(
                            "codemirrorselectwearandroidiconnolabel",
                            **{
                                "value": codemirrorselectwearandroidiconnolabel,
                                "mode": "python",
                                "lineNumbers": True
                            }
                        ),
                        STRONG(I18N("Results"), ":"),
                        DIV(
                            widgets.Select(
                                "selectwearandroidiconnolabel",
                                placeholder="Placeholder Widget",
                                icon=I(_class="fab fa-telegram-plane"),
                                data_set={
                                    "banana": "Banana",
                                    "abacate": "Abacate",
                                    "doce": "Doce"
                                }
                            ),
                            _class="widget_input_example"
                        ),
                        BR(),
                        STRONG(I18N("Code"), ":"),
                        codemirror.CodeMirrorHelper(
                            "codemirrorselectwearandroidicon",
                            **{
                                "value": codemirrorselectwearandroidicon,
                                "mode": "python",
                                "lineNumbers": True
                            }
                        ),
                        STRONG(I18N("Results"), ":"),
                        DIV(
                            widgets.Select(
                                "selectwearandroidicon",
                                label="Label Widget",
                                placeholder="Placeholder Widget",
                                icon=I(_class="fab fa-telegram-plane"),
                                data_set={
                                    "banana": "Banana",
                                    "abacate": "Abacate",
                                    "doce": "Doce"
                                }
                            ),
                            _class="widget_input_example"
                        ),
                        _class="e-padding_10"
                    ),
                ),
                XSECTION(
                    LABEL(I18N("Example"), ' 02 (wear="shadows")'),
                    DIV(
                        STRONG(I18N("Code"), ":"),
                        codemirror.CodeMirrorHelper(
                            "codemirrorselectwearshadowsnolabel",
                            **{
                                "value": codemirrorselectwearshadowsnolabel,
                                "mode": "python",
                                "lineNumbers": True
                            }
                        ),
                        STRONG(I18N("Results"), ":"),
                        DIV(
                            widgets.Select(
                                "selectwearshadowsnolabel",
                                placeholder="Placeholder Widget",
                                data_set={
                                    "banana": "Banana",
                                    "abacate": "Abacate",
                                    "doce": "Doce"
                                },
                                wear="shadows"
                            ),
                            _class="widget_input_example"
                        ),
                        BR(),
                        H3("Label Example"),
                        STRONG(I18N("Code"), ":"),
                        codemirror.CodeMirrorHelper(
                            "codemirrorselectwearshadows",
                            **{
                                "value": codemirrorselectwearshadows,
                                "mode": "python",
                                "lineNumbers": True
                            }
                        ),
                        STRONG(I18N("Results"), ":"),
                        DIV(
                            widgets.Select(
                                "selectwearshadows",
                                label="Label Widget",
                                placeholder="Placeholder Widget",
                                data_set={
                                    "banana": "Banana",
                                    "abacate": "Abacate",
                                    "doce": "Doce"
                                },
                                wear="shadows"
                            ),
                            _class="widget_input_example"
                        ),
                        BR(),
                        STRONG(I18N("Code"), ":"),
                        codemirror.CodeMirrorHelper(
                            "codemirrorselectwearshadowsiconnolabel",
                            **{
                                "value": codemirrorselectwearshadowsiconnolabel,
                                "mode": "python",
                                "lineNumbers": True
                            }
                        ),
                        STRONG(I18N("Results"), ":"),
                        DIV(
                            widgets.Select(
                                "selectwearshadowsiconnolabel",
                                placeholder="Placeholder Widget",
                                icon=I(_class="fab fa-telegram-plane"),
                                data_set={
                                    "banana": "Banana",
                                    "abacate": "Abacate",
                                    "doce": "Doce"
                                },
                                wear="shadows"
                            ),
                            _class="widget_input_example"
                        ),
                        BR(),
                        STRONG(I18N("Code"), ":"),
                        codemirror.CodeMirrorHelper(
                            "codemirrorselectwearshadowsicon",
                            **{
                                "value": codemirrorselectwearshadowsicon,
                                "mode": "python",
                                "lineNumbers": True
                            }
                        ),
                        STRONG(I18N("Results"), ":"),
                        DIV(
                            widgets.Select(
                                "selectwearshadowsicon",
                                label="Label Widget",
                                placeholder="Placeholder Widget",
                                icon=I(_class="fab fa-telegram-plane"),
                                data_set={
                                    "banana": "Banana",
                                    "abacate": "Abacate",
                                    "doce": "Doce"
                                },
                                wear="shadows"
                            ),
                            _class="widget_input_example"
                        ),
                        _class="e-padding_10"
                    ),
                ),
                XSECTION(
                    LABEL(I18N("Example"), ' 03 (wear="elegant")'),
                    DIV(
                        STRONG(I18N("Code"), ":"),
                        codemirror.CodeMirrorHelper(
                            "codemirrorselectwearelegantnolabel",
                            **{
                                "value": codemirrorselectwearelegantnolabel,
                                "mode": "python",
                                "lineNumbers": True
                            }
                        ),
                        STRONG(I18N("Results"), ":"),
                        DIV(
                            widgets.Select(
                                "selectwearelegantnolabel",
                                placeholder="Placeholder Widget",
                                data_set={
                                    "banana": "Banana",
                                    "abacate": "Abacate",
                                    "doce": "Doce"
                                },
                                wear="elegant"
                            ),
                            _class="widget_input_example"
                        ),
                        BR(),
                        STRONG(I18N("Code"), ":"),
                        codemirror.CodeMirrorHelper(
                            "codemirrorselectwearelegant",
                            **{
                                "value": codemirrorselectwearelegant,
                                "mode": "python",
                                "lineNumbers": True
                            }
                        ),
                        STRONG(I18N("Results"), ":"),
                        DIV(
                            widgets.Select(
                                "selectwearelegant",
                                label="Label Widget",
                                placeholder="Placeholder Widget",
                                data_set={
                                    "banana": "Banana",
                                    "abacate": "Abacate",
                                    "doce": "Doce"
                                },
                                wear="elegant"
                            ),
                            _class="widget_input_example"
                        ),
                        BR(),
                        H3("Icon examples"),
                        STRONG(I18N("Code"), ":"),
                        codemirror.CodeMirrorHelper(
                            "codemirrorselectweareleganticonnolabel",
                            **{
                                "value": codemirrorselectweareleganticonnolabel,
                                "mode": "python",
                                "lineNumbers": True
                            }
                        ),
                        STRONG(I18N("Results"), ":"),
                        DIV(
                            widgets.Select(
                                "selectweareleganticonnolabel",
                                placeholder="Placeholder Widget",
                                icon=I(_class="fab fa-telegram-plane"),
                                data_set={
                                    "banana": "Banana",
                                    "abacate": "Abacate",
                                    "doce": "Doce"
                                },
                                wear="elegant"
                            ),
                            _class="widget_input_example"
                        ),
                        H3("Icon example"),
                        STRONG(I18N("Code"), ":"),
                        codemirror.CodeMirrorHelper(
                            "codemirrorselectweareleganticon",
                            **{
                                "value": codemirrorselectweareleganticon,
                                "mode": "python",
                                "lineNumbers": True
                            }
                        ),
                        STRONG(I18N("Results"), ":"),
                        DIV(
                            widgets.Select(
                                "selectweareleganticon",
                                label="Label Widget",
                                placeholder="Placeholder Widget",
                                icon=I(_class="fab fa-telegram-plane"),
                                data_set={
                                    "banana": "Banana",
                                    "abacate": "Abacate",
                                    "doce": "Doce"
                                },
                                wear="elegant"
                            ),
                            _class="widget_input_example"
                        ),
                        _class="e-padding_10"
                    ),
                ),
                H2(I18N("Select - Parameters")),
                HR(),
                XSECTION(
                    LABEL(I18N("Example"), ' 01'),
                    DIV(
                        STRONG(I18N("Code"), ":"),
                        codemirror.CodeMirrorHelper(
                            "codemirrorsimpleselect",
                            **{
                                "value": codemirrorsimpleselect,
                                "mode": "python",
                                "lineNumbers": True
                            }
                        ),
                        STRONG(I18N("Results"), ":"),
                        DIV(
                            widgets.Select(
                                "simpleselect",
                                label="Label Widget",
                                data_set={
                                    "banana": "banana",
                                    "abacate": "abacate",
                                    "doce": "doce"
                                }
                            ),
                            _class="widget_input_example"
                        ),
                        _class="e-padding_10"
                    ),
                ),
                XSECTION(
                    LABEL(I18N("Example"), ' 02 (can_empty="True")'),
                    DIV(
                        STRONG(I18N("Code"), ":"),
                        codemirror.CodeMirrorHelper(
                            "codemirrorselectcanempty",
                            **{
                                "value": codemirrorselectcanempty,
                                "mode": "python",
                                "lineNumbers": True
                            }
                        ),
                        STRONG(I18N("Results"), ":"),
                        DIV(
                            widgets.Select(
                                "selectcanempty",
                                label="Label Widget",
                                can_empty=True,
                                data_set={
                                    "banana": "banana",
                                    "abacate": "abacate",
                                    "doce": "doce"
                                }
                            ),
                            _class="widget_input_example"
                        ),
                        _class="e-padding_10"
                    ),
                ),
                XSECTION(
                    LABEL(I18N("Example"), ' 03 (can_empty=True, placeholder="Placeholder Widget")'),
                    DIV(
                        STRONG(I18N("Code"), ":"),
                        codemirror.CodeMirrorHelper(
                            "codemirrorselectcanemptyplaceholder",
                            **{
                                "value": codemirrorselectcanemptyplaceholder,
                                "mode": "python",
                                "lineNumbers": True
                            }
                        ),
                        STRONG(I18N("Results"), ":"),
                        DIV(
                            widgets.Select(
                                "selectcanemptyplaceholder",
                                label="Label Widget",
                                placeholder="Placeholder Widget",
                                can_empty=True,
                                data_set={
                                    "banana": "banana",
                                    "abacate": "abacate",
                                    "doce": "doce"
                                }
                            ),
                            _class="widget_input_example"
                        ),
                        _class="e-padding_10"
                    ),
                ),
                XSECTION(
                    LABEL(I18N("Example"), ' 04 (editable=True, can_empty=True)'),
                    DIV(
                        STRONG(I18N("Code"), ":"),
                        codemirror.CodeMirrorHelper(
                            "codemirrorselectcanemptyeditable",
                            **{
                                "value": codemirrorselectcanemptyeditable,
                                "mode": "python",
                                "lineNumbers": True
                            }
                        ),
                        STRONG(I18N("Results"), ":"),
                        DIV(
                            widgets.Select(
                                "selectcanemptyeditable",
                                label="Label Widget",
                                editable=True,
                                can_empty=True,
                                data_set={
                                    "banana": "banana",
                                    "abacate": "abacate",
                                    "doce": "doce"
                                }
                            ),
                            _class="widget_input_example"
                        ),
                        _class="e-padding_10"
                    ),
                ),
                XSECTION(
                    LABEL(I18N("Example"),
                        ' 05 (editable=True, can_empty=True, placeholder="Placeholder Widget")'),
                    DIV(
                        STRONG(I18N("Code"), ":"),
                        codemirror.CodeMirrorHelper(
                            "codemirrorselectcanemptyeditableplaceholder",
                            **{
                                "value": codemirrorselectcanemptyeditableplaceholder,
                                "mode": "python",
                                "lineNumbers": True
                            }
                        ),
                        STRONG(I18N("Results"), ":"),
                        DIV(
                            widgets.Select(
                                "selectcanemptyeditableplaceholder",
                                label="Label Widget",
                                placeholder="Placeholder Widget",
                                editable=True,
                                can_empty=True,
                                data_set={
                                    "banana": "banana",
                                    "abacate": "abacate",
                                    "doce": "doce"
                                }
                            ),
                            _class="widget_input_example"
                        ),
                        _class="e-padding_10"
                    ),
                ),
                H2(I18N("MultiSelect")),
                HR(),
                XSECTION(
                    LABEL(I18N("Example"), ' 01'),
                    DIV(
                        STRONG(I18N("Code"), ":"),
                        codemirror.CodeMirrorHelper(
                            "cod_mult_sec_styles",
                            **{
                                "value": cod_mult_sec_styles,
                                "mode": "python",
                                "lineNumbers": True
                            }
                        ),
                        STRONG(I18N("Results"), ":"),
                        H3("Style android"),
                        DIV(
                            widgets.MultSelect(
                                "the_multiselect1stylesnolabelnoplaceholder",
                                data_set={
                                    "banana": "Banana",
                                    "abacate": "Abacate",
                                    "doce": "Doce"
                                },
                            ),
                            widgets.MultSelect(
                                "the_multiselect1stylesnolabelnoplaceh",
                                placeholder="Placeholder Widget",
                                data_set={
                                    "banana": "Banana",
                                    "abacate": "Abacate",
                                    "doce": "Doce"
                                },
                            ),
                            widgets.MultSelect(
                                "the_multiselect1styles",
                                label="Label Widget",
                                placeholder="Placeholder Widget",
                                data_set={
                                    "banana": "Banana",
                                    "abacate": "Abacate",
                                    "doce": "Doce"
                                },
                            ),
                            widgets.MultSelect(
                                "the_multiselect1stylesnolabelicon",
                                placeholder="Placeholder Widget",
                                data_set={
                                    "banana": "Banana",
                                    "abacate": "Abacate",
                                    "doce": "Doce"
                                },
                                icon=I(_class="fab fa-telegram-plane")
                            ),
                            widgets.MultSelect(
                                "the_multiselect1stylesicon",
                                label="Label Widget",
                                placeholder="Placeholder Widget",
                                data_set={
                                    "banana": "Banana",
                                    "abacate": "Abacate",
                                    "doce": "Doce"
                                },
                                icon=I(_class="fab fa-telegram-plane")
                            ),
                            _class="widget_input_example"
                        ),
                        H3("Style shadows"),
                        DIV(
                            widgets.MultSelect(
                                "the_multiselect1stylesshadowsnolabelnoplaceholder",
                                wear="shadows",
                                data_set={
                                    "banana": "Banana",
                                    "abacate": "Abacate",
                                    "doce": "Doce"
                                },
                            ),
                            widgets.MultSelect(
                                "the_multiselect1stylesshadowsnolabelnoplaceh",
                                wear="shadows",
                                placeholder="Placeholder Widget",
                                data_set={
                                    "banana": "Banana",
                                    "abacate": "Abacate",
                                    "doce": "Doce"
                                },
                            ),
                            widgets.MultSelect(
                                "the_multiselect1stylesshadows",
                                label="Label Widget",
                                placeholder="Placeholder Widget",
                                wear="shadows",
                                data_set={
                                    "banana": "Banana",
                                    "abacate": "Abacate",
                                    "doce": "Doce"
                                },
                            ),
                            widgets.MultSelect(
                                "the_multiselect1stylesshadowsnolabelicon",
                                placeholder="Placeholder Widget",
                                wear="shadows",
                                data_set={
                                    "banana": "Banana",
                                    "abacate": "Abacate",
                                    "doce": "Doce"
                                },
                                icon=I(_class="fab fa-telegram-plane")
                            ),
                            widgets.MultSelect(
                                "the_multiselect1stylesshadowsicon",
                                label="Label Widget",
                                placeholder="Placeholder Widget",
                                wear="shadows",
                                data_set={
                                    "banana": "Banana",
                                    "abacate": "Abacate",
                                    "doce": "Doce"
                                },
                                icon=I(_class="fab fa-telegram-plane")
                            ),
                            _class="widget_input_example"
                        ),
                        H3("Style elegant"),
                        DIV(
                            widgets.MultSelect(
                                "the_multiselect1elegantnolabelnoplaceholder",
                                wear="elegant",
                                data_set={
                                    "banana": "Banana",
                                    "abacate": "Abacate",
                                    "doce": "Doce"
                                },
                            ),
                            widgets.MultSelect(
                                "the_multiselect1elegantnolabelnoplaceh",
                                placeholder="Placeholder Widget",
                                wear="elegant",
                                data_set={
                                    "banana": "Banana",
                                    "abacate": "Abacate",
                                    "doce": "Doce"
                                },
                            ),
                            widgets.MultSelect(
                                "the_multiselect1elegant",
                                label="Label Widget",
                                placeholder="Placeholder Widget",
                                wear="elegant",
                                data_set={
                                    "banana": "Banana",
                                    "abacate": "Abacate",
                                    "doce": "Doce"
                                },
                            ),
                            widgets.MultSelect(
                                "the_multiselect1elegantnolabelicon",
                                placeholder="Placeholder Widget",
                                wear="elegant",
                                data_set={
                                    "banana": "Banana",
                                    "abacate": "Abacate",
                                    "doce": "Doce"
                                },
                                icon=I(_class="fab fa-telegram-plane")
                            ),
                            widgets.MultSelect(
                                "the_multiselect1eleganticon",
                                label="Label Widget",
                                placeholder="Placeholder Widget",
                                wear="elegant",
                                data_set={
                                    "banana": "Banana",
                                    "abacate": "Abacate",
                                    "doce": "Doce"
                                },
                                icon=I(_class="fab fa-telegram-plane")
                            ),
                            _class="widget_input_example"
                        ),
                        _class="e-padding_10"
                    ),
                ),
                XSECTION(
                    LABEL(I18N("Example"), ' 02'),
                    DIV(
                        STRONG(I18N("Code"), ":"),
                        codemirror.CodeMirrorHelper(
                            "codemirrormultselect1",
                            **{
                                "value": multselect1_source,
                                "mode": "python",
                                "lineNumbers": True
                            }
                        ),
                        STRONG(I18N("Results"), ":"),
                        DIV(
                            widgets.MultSelect(
                                "the_multiselect1",
                                label="Label Widget",
                                placeholder="Placeholder Widget",
                                data_set={
                                    "banana": "Banana",
                                    "abacate": "Abacate",
                                    "doce": "Doce"
                                },
                            ),
                            _class="widget_input_example"
                        ),
                        _class="e-padding_10"
                    ),
                ),
                XSECTION(
                    LABEL(I18N("Example"), ' 03'),
                    DIV(
                        STRONG(I18N("Code"), ":"),
                        codemirror.CodeMirrorHelper(
                            "codemirrormultselect2",
                            **{
                                "value": multselect2_source,
                                "mode": "python",
                                "lineNumbers": True
                            }
                        ),
                        STRONG(I18N("Results"), ":"),
                        DIV(
                            widgets.MultSelect(
                                "the_multiselect2",
                                label="Label Widget",
                                placeholder="Placeholder Widget",
                                editable=True,
                                data_set={
                                    "banana": "Banana",
                                    "abacate": "Abacate",
                                    "doce": "Doce"
                                },
                            ),
                            _class="widget_input_example"
                        ),
                        _class="e-padding_10"
                    ),
                ),

                _class="card e-padding_20"
            ),
            _class="new-container"
        )
        # jQuery("#main-container").html(html.jquery())
        html.html_to("#selects-container-example")



import phanterpwa.frontend.components.widgets as widgets
import phanterpwa.frontend.helpers as helpers
import plugins.codemirror as codemirror
from org.transcrypt.stubs.browser import __pragma__

__pragma__('alias', "jQuery", "$")
__pragma__('skip')
jQuery = sessionStorage = JSON = M = js_undefined = window =\
    __new__ = FormData = console = localStorage = document = this = 0
__pragma__('noskip')


DIV = helpers.XmlConstructor.tagger("div")
LABEL = helpers.XmlConstructor.tagger("label")
I = helpers.XmlConstructor.tagger("i")
H2 = helpers.XmlConstructor.tagger("h2")
FORM = helpers.XmlConstructor.tagger("form")
SPAN = helpers.XmlConstructor.tagger("span")
IMG = helpers.XmlConstructor.tagger("img", True)
HR = helpers.XmlConstructor.tagger("hr", True)
UL = helpers.XmlConstructor.tagger("ul")
STRONG = helpers.XmlConstructor.tagger("strong")
LI = helpers.XmlConstructor.tagger("li")
INPUT = helpers.XmlConstructor.tagger("input", True)
BR = helpers.XmlConstructor.tagger("br", True)
P = helpers.XmlConstructor.tagger("p")
A = helpers.XmlConstructor.tagger("a")
I18N = helpers.I18N
CONCATENATE = helpers.CONCATENATE
XML = helpers.XML
XSECTION = helpers.XSECTION


codemirrorfirstmenu = """
import phanterpwa.frontend.components.widgets as widgets
import phanterpwa.frontend.helpers as helpers
DIV = helpers.XmlConstructor.tagger("div")

html = DIV(
    widgets.MenuBox(
        "firstmenu",
        I(_class="fas fa-ellipsis-v"),
        DIV("Option1", _class="my_button_one"),
        DIV("Option2", _class="my_button_two")
    )
)

html.html_to("#your_target")
# your code
"""


codemirrormenu2 = """
# your code

your_instance = widgets.MenuBox(
    "firstmenu2",
    "My menu",
    DIV("Option1", _class="my_button_one"),
    DIV("Option2", _class="my_button_two"),
    **{
        "width": 200,
        "onOpen": lambda el: jQuery(el).find(
            ".phanterpwa-widget-menubox-option"
        ).off(
            "click.phanterpwa-awesome-icon",
        ).on(
            "click.phanterpwa-awesome-icon",
            lambda: window.PhanterPWA.flash("I'm clicked!")
        )
    }
)

# your code
"""


codemirrorsidebyside = """
# your code
your_menus = DIV(
    DIV(
        widgets.MenuBox(
            "sideleft",
            I(_class="fas fa-ellipsis-v"),
            DIV("Option1", _class="my_button_one"),
            DIV("Option2", _class="my_button_two")
        ),
        _class="p-col w1p10"
    ),
    DIV(
        " ",
        _class="p-col w1p80"
    ),
    DIV(
        widgets.MenuBox(
            "sideright",
            I(_class="fas fa-ellipsis-v"),
            DIV("Option1", _class="my_button_one"),
            DIV("Option2", _class="my_button_two")
        ),
        _class="p-col w1p10"
    ),
    _class="p-row"
)

# your code
"""


class MenuBox():
    def __init__(self, gatehandler):
        self.gatehandler = gatehandler
        html = CONCATENATE(
            DIV(
                DIV(
                    DIV(
                        DIV("COMPONENTES", _class="phanterpwa-breadcrumb"),
                        DIV("MENUS", _class="phanterpwa-breadcrumb"),
                        _class="phanterpwa-breadcrumb-wrapper"
                    ),
                    _class="p-container"),
                _class='title_page_container card'
            ),
            DIV(
                DIV(
                    DIV(
                        H2(I18N("MenuBox")),
                        HR(),
                        XSECTION(
                            LABEL(I18N("Example"), ' 01'),
                            DIV(
                                STRONG(I18N("Code"), ":"),
                                codemirror.CodeMirrorHelper(
                                    "codemirrorfirstmenu",
                                    **{
                                        "value": codemirrorfirstmenu,
                                        "mode": "python",
                                        "lineNumbers": True
                                    }
                                ),
                                STRONG(I18N("Results"), ":"),
                                DIV(
                                    widgets.MenuBox(
                                        "firstmenu",
                                        I(_class="fas fa-ellipsis-v"),
                                        DIV("Option1", _class="my_button_one"),
                                        DIV("Option2", _class="my_button_two")
                                    ),
                                    _class="widget_input_example"
                                ),
                                _class="e-padding_10"
                            ),
                        ),
                        XSECTION(
                            LABEL(I18N("Example"), ' 02'),
                            DIV(
                                STRONG(I18N("Code"), ":"),
                                codemirror.CodeMirrorHelper(
                                    "codemirrormenu2",
                                    **{
                                        "value": codemirrormenu2,
                                        "mode": "python",
                                        "lineNumbers": True
                                    }
                                ),
                                STRONG(I18N("Results"), ":"),
                                DIV(
                                    widgets.MenuBox(
                                        "firstmenu2",
                                        "My menu",
                                        DIV("Option1", _class="my_button_one"),
                                        DIV("Option2", _class="my_button_two"),
                                        **{
                                            "width": 200,
                                            "onOpen": lambda el: jQuery(el).find(
                                                ".phanterpwa-widget-menubox-option"
                                            ).off(
                                                "click.phanterpwa-awesome-icon",
                                            ).on(
                                                "click.phanterpwa-awesome-icon",
                                                lambda: window.PhanterPWA.flash("I'm clicked!")
                                            )
                                        }
                                    ),
                                    _class="widget_input_example"
                                ),
                                _class="e-padding_10"
                            ),
                        ),
                        XSECTION(
                            LABEL(I18N("Example"), ' 01 (wear="android")'),
                            DIV(
                                STRONG(I18N("Code"), ":"),
                                codemirror.CodeMirrorHelper(
                                    "codemirrorsidebyside",
                                    **{
                                        "value": codemirrorsidebyside,
                                        "mode": "python",
                                        "lineNumbers": True
                                    }
                                ),
                                STRONG(I18N("Results"), ":"),
                                DIV(
                                    DIV(
                                        DIV(
                                            widgets.MenuBox(
                                                "sideleft",
                                                I(_class="fas fa-ellipsis-v"),
                                                DIV("Option1", _class="my_button_one"),
                                                DIV("Option2", _class="my_button_two")
                                            ),
                                            _class="p-col w1p10"
                                        ),
                                        DIV(
                                            " ",
                                            _class="p-col w1p80"
                                        ),
                                        DIV(
                                            widgets.MenuBox(
                                                "sideright",
                                                I(_class="fas fa-ellipsis-v"),
                                                DIV("Option1", _class="my_button_one"),
                                                DIV("Option2", _class="my_button_two")
                                            ),
                                            _class="p-col w1p10"
                                        ),
                                        _class="p-row"
                                    ),
                                    _class="widget_input_example"
                                ),
                                _class="e-padding_10"
                            ),
                        ),



                        _class="card e-padding_20"
                    ),
                    _class="new-container"
                ),
                _class="phanterpwa-container p-container"
            )
        )
        # jQuery("#main-container").html(html.jquery())
        html.html_to("#main-container")



import os
import re
import ast
import json
import hashlib
from phanterpwa.backend.decorators import (
    requires_authentication,
    check_private_csrf_token
)
from passlib.hash import pbkdf2_sha512
from pydal.objects import (
    Field
)
from pydal.validators import (
    IS_EMAIL,
    IS_NOT_EMPTY,
    IS_NOT_IN_DB
)
from phanterpwa.i18n import browser_language
from tornado import (
    web
)
from phanterpwa.mail import MailSender
from phanterpwa.samples import (
    email_activation_code,
    email_password
)
from datetime import (
    datetime,
    timedelta
)
from phanterpwa.tools import (
    humanize_seconds,
    temporary_password,
    interpolate,
    generate_activation_code,
    checkbox_bool,
    string_escape as E
)
from phanterpwa.backend.dataforms import (
    FieldsDALValidateDictArgs,
    FormFromTableDAL as jsonForm
)

from phanterpwa.backend.request_handlers.auth import (
    arbritary_login
)
from phanterpwa.backend.request_handlers.websocket import (
    EchoWebSocket
)
from phanterpwa.gallery.cutter import PhanterpwaGalleryCutter
from phanterpwa.gallery.integrationDAL import PhanterpwaGalleryUserImage
new_group_re = re.compile(r"\$[0-9]{13}\:(.{0,})")


class UserManager(web.RequestHandler):
    """
        url: '/api/admin/usermanager/<id_user>'
    """

    def initialize(self, app_name, projectConfig, DALDatabase, i18nTranslator=None, logger_api=None):
        self.app_name = app_name
        self.projectConfig = projectConfig
        self.DALDatabase = DALDatabase
        self.i18nTranslator = i18nTranslator
        if logger_api:
            self.logger_api = logger_api
        if i18nTranslator:
            self.T = i18nTranslator.T
        self.set_header("Access-Control-Allow-Origin", "*")
        self.set_header(
            "Access-Control-Allow-Headers",
            "".join([
                "phanterpwa-language,",
                "phanterpwa-application,",
                "phanterpwa-application-version,",
                "phanterpwa-client-token,",
                "phanterpwa-authorization,",
                "cache-control"
            ])
        )
        self.set_header('Access-Control-Allow-Methods', 'GET, OPTIONS, PUT, DELETE')
        if self.request.headers.get("phanterpwa-language"):
            self.phanterpwa_language = self.request.headers.get("phanterpwa-language")
        else:
            self.phanterpwa_language = browser_language(self.request.headers.get("Accept-Language"))
        if self.i18nTranslator:
            self.i18nTranslator.direct_translation = self.phanterpwa_language
        self.phanterpwa_user_agent = str(self.request.headers.get('User-Agent'))
        self.phanterpwa_remote_ip = str(self.request.remote_ip)

    def check_origin(self, origin):
        return True

    def _request_summary(self) -> str:
        client_ip = self.request.headers.get('X-Real-IP') or\
            self.request.headers.get('X-Forwarded-For', '').split(',')[0].strip() or\
            self.request.remote_ip
        summary = "{0} {1} ({2})".format(
            self.request.method,
            self.request.uri,
            client_ip,
        )
        if hasattr(self, "phanterpwa_current_user") and self.phanterpwa_current_user is not None:
            summary = "{0} {1} ({2} - {3})".format(
                self.request.method,
                self.request.uri,
                client_ip,
                self.phanterpwa_current_user.email
            )
        return summary

    def options(self, *args):
        self.set_status(200)
        self.write({"status": "OK"})

    @requires_authentication(roles_name="root")
    def get(self, *args, **kargs):
        dict_arguments = {k: self.request.arguments.get(k)[0].decode('utf-8') for k in self.request.arguments}
        id_user = args[0]
        action = args[1]
        db = self.DALDatabase
        remote_adress = self.projectConfig["BACKEND"]["api"]["http_address"]
        if id_user:
            if id_user == "new":
                msg = "New User"

                json_form = jsonForm(db.auth_user)
                json_form.add_extra_field(
                    Field('foto3x4', phanterpwa={
                        'type': 'image',
                        'position': 0,
                        'url': lambda row: "{0}/api/socios/{1}/image/".format(
                            remote_adress,
                            row.id
                        ),
                        'cutter': True,
                        'no-cache': True,
                        'group': 'group1',
                        '_class': 'p-col w1p100 w4p30'
                    })
                )
                auth_group = db(
                    (db.auth_group.role == "user")
                ).select(db.auth_group.ALL)
                json_form.add_extra_field(
                    Field('auth_group', label="Roles", phanterpwa={
                        "value": [
                            [x.id, "{0} - {1}".format(
                                x.grade, x.role)] for x in auth_group
                        ],
                        "data_set": [
                            [x.id, "{0} - {1}".format(
                                x.grade, x.role)] for x in db(db.auth_group).select(db.auth_group.ALL)
                        ],
                        "form": "auth_user",
                        "type": 'list_string',
                        "section": "List of user roles ",
                        "_class": "p-col w1p100"
                    })
                )
                json_form = json_form.as_dict()
                to_write = {
                    "status": "OK",
                    "code": 200,
                    "message": msg,
                    "data": {
                        "auth_user": json_form
                    }
                }
                md5 = hashlib.md5()
                md5.update(str(to_write).encode("utf-8"))
                md5 = md5.hexdigest()
                to_write["hash"] = md5
                return self.write(to_write)
            elif action == "edit" or action == "view":
                q_auth_user = db(db.auth_user.id == id_user).select(db.auth_user.id).first()
                q_gallery = db(db.auth_user_phanterpwagallery.auth_user == id_user).select().first()
                if q_auth_user:
                    msg = "User (id:{0})".format(id_user)
                    json_form = jsonForm(db.auth_user, q_auth_user.id)
                    json_form.add_extra_field(
                        Field('foto3x4', phanterpwa={
                            'type': 'image',
                            'position': 0,
                            'url': lambda row: "{0}/api/admin/usermanager/{1}/image/".format(
                                remote_adress,
                                row.id
                            ),
                            'cutter': True,
                            'no-cache': True,
                            'group': 'group1',
                            '_class': 'p-col w1p100 w4p30'
                        })
                    )
                    auth_group = db(
                        (db.auth_membership.auth_user == id_user) &
                        (db.auth_group.id == db.auth_membership.auth_group)
                    ).select(db.auth_group.ALL)
                    json_form.add_extra_field(
                        Field('auth_group', label="Roles", phanterpwa={
                            "value": [
                                [x.id, "{0} - {1}".format(
                                    x.grade, x.role)] for x in auth_group
                            ],
                            "data_set": [
                                [x.id, "{0} - {1}".format(
                                    x.grade, x.role)] for x in db(db.auth_group).select(db.auth_group.ALL)
                            ],
                            "form": "auth_user",
                            "type": 'list_string',
                            "section": "List of user roles ",
                            "_class": "p-col w1p100"
                        })
                    )
                    json_form = json_form.as_dict()

                    to_write = {
                        "status": "OK",
                        "code": 200,
                        "message": msg,
                        "data": {
                            "gallery": json.loads(q_gallery.as_json()) if q_gallery else None,
                            "auth_user": json_form
                        }
                    }
                    md5 = hashlib.md5()
                    md5.update(str(to_write).encode("utf-8"))
                    md5 = md5.hexdigest()
                    to_write["hash"] = md5
                    return self.write(to_write)
                else:
                    msg = "User not found"
                    self.set_status(404)
                    return self.write({
                        "status": "Not Found",
                        "code": 404,
                        "message": msg,
                        "auth_user": None
                    })

        else:
            limit = 100
            p_inicial = 0
            search = dict_arguments.get("search", "")
            search_field = dict_arguments.get("field", "id")
            orderby = dict_arguments.get("orderby", "id")
            sort = dict_arguments.get("sort", "asc")
            page = dict_arguments.get("page", "1")
            order_list = ["id", "first_name", "last_name", "email",
                "permit_mult_login", "activated"]
            if orderby in order_list:
                _orderby = db.auth_user[orderby]
            else:
                _orderby = db.auth_user.id

            if sort == "desc":
                _orderby = ~_orderby
            conns = EchoWebSocket.get_connections()
            onlines = [conn.phanterpwa_current_user.id for conn in conns if conn.phanterpwa_current_user is not None]
            query = db.auth_user
            msg = "Users"
            if search and search_field:
                msg = "Search by \"{0}\" on \"{1}\"".format(search, search_field)
                if search_field in ["first_name", "last_name", "email"]:
                    query = db.auth_user[search_field].contains(search)
                elif search_field in ["activated", "permit_mult_login"]:
                    query = db.auth_user[search_field] == checkbox_bool(search)
                elif search_field in ["id", "grade"] and search.isdigit():
                    query = db.auth_group[search_field] == int(search)
                elif search_field in order_list:
                    query = db.auth_user[search_field] == search
                elif search_field == "websocket_opened":
                    if checkbox_bool(search):
                        query = db.auth_user.id.belongs(onlines)
                    else:
                        query = ~db.auth_user.id.belongs(onlines)


            t_users = db(query).count()
            displayed_records = t_users
            if t_users > limit:
                displayed_records = limit

            if (t_users % limit) == 0:
                total_pages = t_users / limit
            else:
                total_pages = (t_users // limit) + 1

            if page.isdigit():
                page = int(page)
                if page > 1:
                    if page > total_pages:
                        page = total_pages

                    p_inicial = (page - 1) * limit
            else:
                page = 1

            q_users = db(query).select(
                *[db.auth_user[x] for x in order_list],
                orderby=_orderby, limitby=(p_inicial, p_inicial + limit)
            )

            users = {
                'searcher': {
                    'search': search,
                    'page': page,
                    'total_pages': total_pages,
                    'total_records': t_users,
                    'displayed_records': displayed_records,
                    'field': search_field,
                    'sort_by': orderby,
                    'sort_order': sort,
                    'data_set': [
                        ["id", "ID"],
                        ["first_name", "First Name"],
                        ["last_name", "Last Name"],
                        ['email', "E-mail"],
                        ["permit_mult_login", "Allows Multiple Logins"],
                        ["activated", "Activated"],
                        ["websocket_opened", "Online"]
                        
                    ],
                    'sortable': order_list
                },
                'search_fields': {
                    'label': "Search fields",
                    'value': search_field,
                    'data_set': [
                        ["id", "ID"],
                        ["first_name", "First Name"],
                        ["last_name", "Last Name"],
                        ['email', "E-mail"],
                        ["permit_mult_login", "Allows Multiple Logins"],
                        ["activated", "Activated"],
                        ["websocket_opened", "Online"]
                    ]
                },
                'vars': dict_arguments
            }

            if q_users:
                users['data'] = []
                for user in q_users:
                    us = json.loads(user.as_json())
                    us["websocket_opened"] = True if user.id in onlines else False
                    users['data'].append(us)
            else:
                msg = "No record found"

            to_write = {
                "status": "OK",
                "uri": self.request.uri,
                "code": 200,
                "message": msg,
                "users": users if users else None
            }
            md5 = hashlib.md5()
            md5.update(str(to_write).encode("utf-8"))
            md5 = md5.hexdigest()
            to_write["hash"] = md5
            self.set_status(200)
            return self.write(to_write)

    @check_private_csrf_token(form_identify=["phanterpwa-form-auth_user"])
    @requires_authentication(roles_name="root")
    def put(self, *args, **kargs):
        db = self.DALDatabase
        id_user = args[0]
        q_user = db(db.auth_user.id == id_user).select()
        if not q_user:
            message = "The id user not exist."
            self.set_status(400)
            return self.write({
                'status': 'Bad Request',
                'code': 400,
                'message': message,
                'i18n': {
                    'message': self.T(message)
                }
            })
        else:
            r_user = q_user.first()
        dict_arguments = {k: self.request.arguments.get(k)[0].decode('utf-8') for k in self.request.arguments}
        dict_arguments['id'] = id_user
        first_name = dict_arguments.get('first_name', r_user.first_name)
        last_name = dict_arguments.get('last_name', r_user.last_name)
        email_now = r_user.email
        new_email = dict_arguments['email']
        two_factor = checkbox_bool(dict_arguments.get('two_factor', r_user.two_factor_login))
        multiple_login = checkbox_bool(dict_arguments.get('permit_mult_login', r_user.permit_mult_login))
        activated = checkbox_bool(dict_arguments.get('activated', r_user.activated))

        db.auth_user.email.requires = [IS_EMAIL()]
        table = db.auth_user
        result = FieldsDALValidateDictArgs(
            dict_arguments,
            *[table[x] for x in table.fields if x in ["first_name", "last_name"]],
            table["email"] if new_email != email_now else None
        )
        r = result.validate()
        if r:
            message = 'The form has errors'
            i18n_errors = {}
            for x in result.errors:
                tran = self.T(result.errors[x])
                i18n_errors[x] = tran
            self.set_status(400)
            return self.write({
                'status': 'Bad Request',
                'code': 400,
                'message': message,
                'errors': result.errors,
                'i18n': {
                    'message': self.T(message),
                    'errors': i18n_errors
                }
            })
        else:
            if dict_arguments.get("auth_group", None):
                try:
                    auth_group = ast.literal_eval(json.loads(dict_arguments["auth_group"]))
                except Exception:
                    auth_group = ast.literal_eval(dict_arguments["auth_group"])
                if isinstance(auth_group, list):
                    ids_groups = []
                    for x in auth_group:
                        role = None
                        if new_group_re.match(x):
                            role = new_group_re.findall(x)[0]
                            q_group = db(db.auth_group.role == role).select()
                            if q_group:
                                ids_groups.append(q_group.first().id)
                            else:
                                id_new_group = db.auth_group.insert(
                                    role=role,
                                    grade=2
                                )
                                ids_groups.append(id_new_group)
                        elif str(x).isdigit():
                            ids_groups.append(int(x))
                    if ids_groups:
                        db(
                            (db.auth_membership.auth_user == id_user)
                            & (~db.auth_membership.auth_group.belongs(ids_groups))
                        ).delete()
                        q_auth_membership = db(
                            (db.auth_membership.auth_user == id_user)
                            & (db.auth_membership.auth_group.belongs(ids_groups))
                        ).select()
                        novos_dependentes = set(ids_groups).difference(set([x.auth_group for x in q_auth_membership]))
                        for x in novos_dependentes:
                            db.auth_membership.insert(
                                auth_user=id_user,
                                auth_group=x
                            )
                    else:
                        db(
                            (db.auth_membership.auth_user == id_user)
                        ).delete()
                    db.commit()

            email_change = False
            first_name_change = False
            last_name_change = False
            image_change = False
            two_factor_change = False
            multiple_login_change = False
            activated_change = False

            if(first_name != r_user.first_name):
                r_user.update_record(first_name=first_name)
                first_name_change = True

            if(last_name != r_user.last_name):
                r_user.update_record(last_name=last_name)
                last_name_change = True

            if(two_factor != r_user.two_factor_login):
                r_user.update_record(two_factor_login=two_factor)
                two_factor_change = True

            if(multiple_login != r_user.permit_mult_login):
                r_user.update_record(permit_mult_login=multiple_login)
                multiple_login_change = True

            if(activated != r_user.activated):
                r_user.update_record(activated=activated)
                activated_change = True


            if self.request.files and\
                "phanterpwa-gallery-file-input" in self.request.files:
                imageBytes = self.request.files["phanterpwa-gallery-file-input"][0]['body']
                filename = self.request.files["phanterpwa-gallery-file-input"][0]['filename']
                cutterSizeX = dict_arguments['phanterpwa-gallery-input-cutterSizeX']
                cutterSizeY = dict_arguments['phanterpwa-gallery-input-cutterSizeY']
                cut_file = PhanterpwaGalleryCutter(
                    imageName=filename,
                    imageBytes=imageBytes,
                    cutterSizeX=cutterSizeX,
                    cutterSizeY=cutterSizeY
                )
                if 'phanterpwa-gallery-input-autoCut' in dict_arguments and\
                    dict_arguments['phanterpwa-gallery-input-autoCut']:
                    cutedImage = cut_file.auto_cut()
                else:
                    positionX = dict_arguments['phanterpwa-gallery-input-positionX']
                    positionY = dict_arguments['phanterpwa-gallery-input-positionY']
                    newSizeX = dict_arguments['phanterpwa-gallery-input-newSizeX']
                    newSizeY = dict_arguments['phanterpwa-gallery-input-newSizeY']
                    cutedImage = cut_file.specific_cut(
                        newSizeX=newSizeX,
                        newSizeY=newSizeY,
                        positionX=positionX,
                        positionY=positionY
                    )
                upload_image = PhanterpwaGalleryUserImage(
                    id_user,
                    self.DALDatabase,
                    self.projectConfig
                )
                upload_image.set_image(
                    *cutedImage
                )
            activate = r_user.activated
            if new_email != email_now:
                activation_code = generate_activation_code()
                keys_formatter = dict(
                    app_name=self.projectConfig['PROJECT']['name'],
                    user_name="{0} {1}".format(
                        first_name,
                        last_name
                    ),
                    code=activation_code,
                    time_expires=humanize_seconds(
                        self.projectConfig['BACKEND'][self.app_name]['default_time_activation_code_expire']
                    ),
                    copyright=interpolate(self.projectConfig['CONTENT_EMAILS']['copyright'], {'now': datetime.now().year}),
                    link_to_your_page=self.projectConfig['CONTENT_EMAILS']['link_to_your_site']
                )
                email_password.text.formatter(keys_formatter)
                text_email = email_activation_code.text.html(
                    minify=True,
                    translate=True,
                    formatter=keys_formatter,
                    do_not_translate=["\n", " ", "\n\n", "&nbsp;"],
                    escape_string=False
                )
                html_email = email_activation_code.html.html(
                    minify=True,
                    translate=True,
                    formatter=keys_formatter,
                    do_not_translate=["\n", " ", "\n\n", "&nbsp;"],
                    escape_string=False
                )

                e_mail = MailSender(
                    self.projectConfig['EMAIL']['default_sender'],
                    self.projectConfig['EMAIL']['password'],
                    new_email,
                    subject="Activation code",
                    text_message=text_email,
                    html_message=html_email,
                    server=self.projectConfig['EMAIL']['server'],
                    port=self.projectConfig['EMAIL']['port'],
                    use_tls=self.projectConfig['EMAIL']['use_tls'],
                    use_ssl=self.projectConfig['EMAIL']['use_ssl']
                )
                result = ""
                try:
                    if self.projectConfig["PROJECT"]["debug"]:
                        self.logger_api.warning("ACTIVATION CODE: {0}".format(activation_code))
                    else:
                        self.logger_api.warning("Email from '{0}' to '{1}' -> Activation Code: {2}".format(
                            self.projectConfig['EMAIL']['default_sender'],
                            new_email,
                            activation_code
                        ))
                        e_mail.send()
                except Exception as e:
                    result = "Email from '{0}' to '{1}' don't send! -> Error: {2} -> password: {3}".format(
                        self.projectConfig['EMAIL']['default_sender'], dict_arguments['email'], e, activation_code)
                    self.logger_api.error(result, exc_info=True)
                    message = "There was an error trying to send the email."
                    message_i18n = self.T("There was an error trying to send the email.")
                    self.DALDatabase.rollback()
                    self.set_status(400)
                    return self.write({
                        'status': 'Bad Request',
                        'code': 400,
                        'message': message,
                        'i18n': {'message': message_i18n}
                    })
                else:
                    r_user.update_record(
                        activation_code=activation_code.split("-")[0],
                        timeout_to_resend_activation_email=datetime.now() +
                            timedelta(seconds=self.projectConfig['BACKEND'][self.app_name]['default_time_activation_code_expire'])
                    )
                    activate = False
                    r_user.update_record(email=new_email, activated=activate)
                    q_list = self.DALDatabase(
                        (self.DALDatabase.email_user_list.auth_user == id_user) &
                        (self.DALDatabase.email_user_list.email == email_now)
                    ).select().first()
                    if q_list:
                        q_list.update_record(
                            datetime_changed=datetime.now()
                        )
                    else:
                        self.DALDatabase.email_user_list.insert(
                            auth_user=id_user,
                            email=email_now
                        )
                    email_change = True

            self.DALDatabase(
                (self.DALDatabase.client.auth_user == id_user)
            ).delete()
            self.DALDatabase.commit()
            self.set_status(200)
            return self.write({
                'status': 'OK',
                'code': 200,
                'message': 'Account was successfully changed',
                'i18n': {
                    'message': self.T('Account was successfully changed'),
                }
            })

    @requires_authentication(roles_name="root")
    def delete(self, *args, **kargs):
        db = self.DALDatabase
        id_user = args[0]
        q_user = db(db.auth_user.id == id_user).select().first()
        if not q_user:
            message = "The id user not exist."
            self.set_status(400)
            return self.write({
                'status': 'Bad Request',
                'code': 400,
                'message': message,
                'i18n': {
                    'message': self.T(message)
                }
            })
        else:
            db(db.auth_user.id == q_user.id).delete()
            db.commit()
            self.set_status(200)
            return self.write({
                'status': 'OK',
                'code': 200,
                'message': 'The account was successfully deleted',
                'i18n': {
                    'message': self.T('The account was successfully deleted'),
                }
            })


    def options(self, *args):
        self.set_status(200)
        self.write({
            "status": "OK",
        })


class UserImage(web.RequestHandler):
    """
        url: '/api/admin/usermanager/<id_socio>/image/'
    """

    def initialize(self, *args, **kargs):
        self.app_name = kargs.get("app_name", None)
        self.projectConfig = kargs.get("projectConfig", None)
        self.DALDatabase = kargs.get("DALDatabase", None)
        self.i18nTranslator = kargs.get("i18nTranslator", None)
        logger_api = kargs.get("logger_api", None)
        if logger_api:
            self.logger_api = logger_api
        if self.i18nTranslator:
            self.T = self.i18nTranslator.T
        self.set_header("Access-Control-Allow-Origin", "*")
        self.set_header(
            "Access-Control-Allow-Headers",
            "".join([
                "phanterpwa-language,",
                "phanterpwa-application,",
                "phanterpwa-application-version,",
                "phanterpwa-client-token,",
                "cache-control"
            ])
        )
        self.set_header('Access-Control-Allow-Methods', 'GET, OPTIONS')
        if self.request.headers.get("phanterpwa-language"):
            self.phanterpwa_language = self.request.headers.get("phanterpwa-language")
        else:
            self.phanterpwa_language = browser_language(self.request.headers.get("Accept-Language"))
        if self.i18nTranslator:
            self.i18nTranslator.direct_translation = self.phanterpwa_language

    def check_origin(self, origin):
        return True

    def _request_summary(self) -> str:
        client_ip = self.request.headers.get('X-Real-IP') or\
            self.request.headers.get('X-Forwarded-For', '').split(',')[0].strip() or\
            self.request.remote_ip
        summary = "{0} {1} ({2})".format(
            self.request.method,
            self.request.uri,
            client_ip,
        )
        if hasattr(self, "phanterpwa_current_user") and self.phanterpwa_current_user is not None:
            summary = "{0} {1} ({2} - {3})".format(
                self.request.method,
                self.request.uri,
                client_ip,
                self.phanterpwa_current_user.email
            )
        return summary

    def options(self, *args):
        self.set_status(200)
        return self.write({"status": "OK"})

    #@check_url_token()
    def get(self, *args, **kargs):
        db = self.DALDatabase
        projectConfig = self.projectConfig
        """
        Receive request to create and response with a token csrf or captcha
        """
        id_user = args[0]

        buf_size = 512
        q_image = db(
            (db.auth_user_phanterpwagallery.phanterpwagallery == db.phanterpwagallery.id)
            & (db.auth_user_phanterpwagallery.auth_user == id_user)
        ).select(
            db.phanterpwagallery.folder,
            db.phanterpwagallery.alias_name,
            db.phanterpwagallery.filename
        ).last()
        if q_image:
            file = os.path.join(projectConfig['PROJECT']['path'],
                    'backapps', 'api', 'uploads', q_image.folder,
                        q_image.alias_name)
            self.set_header(
                'Content-Disposition', 'attachment; filename="{0}"'.format(
                    q_image.filename)
            )
            if os.path.isfile(file):
                self.set_status(200)
                with open(file, 'rb') as f:
                    while True:
                        data = f.read(buf_size)
                        if not data:
                            break
                        self.write(data)
                self.finish()
                return
        self.set_status(202)
        file = os.path.join(self.projectConfig['PROJECT']['path'],
                "backapps", self.app_name, 'statics', 'images', 'user.png')
        with open(file, 'rb') as f:
            while True:
                data = f.read(buf_size)
                if not data:
                    break
                self.write(data)
        self.finish()
        return


class RoleManager(web.RequestHandler):
    """
        url: '/api/admin/rolemanager/<id_role>'
    """

    def initialize(self, app_name, projectConfig, DALDatabase, i18nTranslator=None, logger_api=None):
        self.app_name = app_name
        self.projectConfig = projectConfig
        self.DALDatabase = DALDatabase
        self.i18nTranslator = i18nTranslator
        if logger_api:
            self.logger_api = logger_api
        if i18nTranslator:
            self.T = i18nTranslator.T
        self.set_header("Access-Control-Allow-Origin", "*")
        self.set_header(
            "Access-Control-Allow-Headers",
            "".join([
                "phanterpwa-language,",
                "phanterpwa-application,",
                "phanterpwa-application-version,",
                "phanterpwa-client-token,",
                "phanterpwa-authorization,",
                "cache-control"
            ])
        )
        self.set_header('Access-Control-Allow-Methods', 'GET, OPTIONS, POST, PUT')
        if self.request.headers.get("phanterpwa-language"):
            self.phanterpwa_language = self.request.headers.get("phanterpwa-language")
        else:
            self.phanterpwa_language = browser_language(self.request.headers.get("Accept-Language"))
        if self.i18nTranslator:
            self.i18nTranslator.direct_translation = self.phanterpwa_language
        self.phanterpwa_user_agent = str(self.request.headers.get('User-Agent'))
        self.phanterpwa_remote_ip = str(self.request.remote_ip)

    def check_origin(self, origin):
        return True

    def _request_summary(self) -> str:
        client_ip = self.request.headers.get('X-Real-IP') or\
            self.request.headers.get('X-Forwarded-For', '').split(',')[0].strip() or\
            self.request.remote_ip
        summary = "{0} {1} ({2})".format(
            self.request.method,
            self.request.uri,
            client_ip,
        )
        if hasattr(self, "phanterpwa_current_user") and self.phanterpwa_current_user is not None:
            summary = "{0} {1} ({2} - {3})".format(
                self.request.method,
                self.request.uri,
                client_ip,
                self.phanterpwa_current_user.email
            )
        return summary

    def options(self, *args):
        self.set_status(200)
        self.write({"status": "OK"})

    @requires_authentication(roles_name="root")
    def get(self, *args, **kargs):
        dict_arguments = {k: self.request.arguments.get(k)[0].decode('utf-8') for k in self.request.arguments}
        id_group = args[0]
        action = args[1]
        db = self.DALDatabase

        if id_group:
            if id_group == "new":
                msg = "New Role"

                json_form = jsonForm(db.auth_group)
                json_form = json_form.as_dict()
                to_write = {
                    "status": "OK",
                    "code": 200,
                    "message": msg,
                    "data": {
                        "auth_role": json_form
                    }
                }
                md5 = hashlib.md5()
                md5.update(str(to_write).encode("utf-8"))
                md5 = md5.hexdigest()
                to_write["hash"] = md5
                return self.write(to_write)
            elif action == "edit" or action == "view":
                q_auth_role = db(db.auth_group.id == id_group).select(db.auth_group.id).first()
                if q_auth_role:
                    msg = "Role (id:{0})".format(id_group)
                    json_form = jsonForm(db.auth_group, q_auth_role.id)
                    json_form = json_form.as_dict()

                    to_write = {
                        "status": "OK",
                        "code": 200,
                        "message": msg,
                        "data": {
                            "auth_role": json_form
                        }
                    }
                    md5 = hashlib.md5()
                    md5.update(str(to_write).encode("utf-8"))
                    md5 = md5.hexdigest()
                    to_write["hash"] = md5
                    return self.write(to_write)
                else:
                    msg = "Role not found"
                    self.set_status(404)
                    return self.write({
                        "status": "Not Found",
                        "code": 404,
                        "message": msg,
                        "auth_role": None
                    })

        else:
            limit = 100
            p_inicial = 0
            search = dict_arguments.get("search", "")
            search_field = dict_arguments.get("field", "grade")
            orderby = dict_arguments.get("orderby", "grade")
            sort = dict_arguments.get("sort", "asc")
            page = dict_arguments.get("page", "1")
            order_list = ["id", "grade", "role", "description"]

            if orderby in order_list:
                _orderby = db.auth_group[orderby]
            else:
                _orderby = db.auth_group.id

            if sort == "desc":
                _orderby = ~_orderby

            query = db.auth_group
            msg = "Roles"
            if search and search_field and (search_field in order_list):
                msg = "Search by \"{0}\" on \"{1}\"".format(search, search_field)
                if search_field in ["role", "description"]:
                    query = db.auth_group[search_field].contains(search)
                elif search_field in ["id", "grade"] and search.isdigit():
                    query = db.auth_group[search_field] == int(search)

            t_groups = db(query).count()
            displayed_records = t_groups
            if t_groups > limit:
                displayed_records = limit

            if (t_groups % limit) == 0:
                total_pages = t_groups / limit
            else:
                total_pages = (t_groups // limit) + 1

            if page.isdigit():
                page = int(page)
                if page > 1:
                    if page > total_pages:
                        page = total_pages

                    p_inicial = (page - 1) * limit
            else:
                page = 1

            q_groups = db(query).select(
                *[db.auth_group[x] for x in order_list],
                orderby=_orderby, limitby=(p_inicial, p_inicial + limit)
            )

            groups = {
                'searcher': {
                    'search': search,
                    'page': page,
                    'total_pages': total_pages,
                    'total_records': t_groups,
                    'displayed_records': displayed_records,
                    'field': search_field,
                    'sort_by': orderby,
                    'sort_order': sort,
                    'data_set': [
                        ["id", "ID"],
                        ["grade", "Grade"],
                        ["role", "Role"],
                        ["description", "Description"]
                    ],
                    'sortable': order_list
                },
                'search_fields': {
                    'label': "Search fields",
                    'value': search_field,
                    'data_set': [
                        ["id", "ID"],
                        ["grade", "Grade"],
                        ["role", "Role"],
                        ["description", "Description"]
                    ]
                },
                'vars': dict_arguments
            }

            if q_groups:
                groups['data'] = json.loads(q_groups.as_json())
            else:
                msg = "No record found"

        to_write = {
            "status": "OK",
            "uri": self.request.uri,
            "code": 200,
            "message": msg,
            "groups": groups if groups else None
        }
        md5 = hashlib.md5()
        md5.update(str(to_write).encode("utf-8"))
        md5 = md5.hexdigest()
        to_write["hash"] = md5
        self.set_status(200)
        return self.write(to_write)

    @check_private_csrf_token(form_identify=["phanterpwa-form-auth_group"])
    @requires_authentication(roles_name="root")
    def post(self, *args, **kargs):
        db = self.DALDatabase

        dict_arguments = {k: self.request.arguments.get(k)[0].decode('utf-8') for k in self.request.arguments}
        grade = dict_arguments.get("grade", 0)
        grade_error = None
        if str(grade).isdigit() and int(grade) < 99:
            dict_arguments['grade'] = int(grade)
        else:
            grade_error = "The grade must be an integer and less than 99"


        db.auth_group.role.requires = [IS_NOT_EMPTY(), IS_NOT_IN_DB(db, db.auth_group.role)]
        table = db.auth_group
        result = FieldsDALValidateDictArgs(
            dict_arguments,
            *[table[x] for x in table.fields]
        )
        r = result.validate()
        if r or grade_error:
            if r and grade_error:
                result.errors["grade"] = grade_error
                data_error = result.errors
            elif grade_error:
                data_error = {"grade": grade_error}
            else:
                data_error = result.errors

            i18n_errors = {}
            for x in data_error:
                tran = self.T(data_error[x])
                i18n_errors[x] = tran
            message = 'The form has errors'
            self.set_status(400)
            return self.write({
                'status': 'Bad Request',
                'code': 400,
                'message': message,
                'errors': data_error,
                'i18n': {
                    'message': self.T(message),
                    'errors': i18n_errors
                }
            })
        else:
            id_record = result.insert(table)
            q_role = db(db.auth_group.id == id_record).select().first()
            if q_role:
                self.set_status(200)
                db.commit()
                return self.write({
                    'status': 'OK',
                    'code': 200,
                    'message': 'Group was successfully changed',
                    'auth_group': {
                        'id': str(q_role.id),
                        'role': E(q_role.role),
                        'grade': q_role.grade,
                        'description': E(q_role.description)
                    },
                    'i18n': {
                        'message': self.T('Group was successfully changed'),
                        'auth_role': {'role': self.T(q_role.role)}
                    }
                })
        self.set_status(400)
        return self.write({
            'status': 'Bad Request',
            'code': 400,

        })

    @check_private_csrf_token(form_identify=["phanterpwa-form-auth_group"])
    @requires_authentication(roles_name="root")
    def put(self, *args, **kargs):
        db = self.DALDatabase
        id_role = args[0]
        q_role = db(db.auth_group.id == id_role).select()
        if not q_role:
            message = "The id role not exist."
            self.set_status(400)
            return self.write({
                'status': 'Bad Request',
                'code': 400,
                'message': message,
                'i18n': {
                    'message': self.T(message)
                }
            })
        else:
            q_role = q_role.first()
        dict_arguments = {k: self.request.arguments.get(k)[0].decode('utf-8') for k in self.request.arguments}
        dict_arguments['id'] = id_role

        table = db.auth_group
        result = FieldsDALValidateDictArgs(
            dict_arguments,
            *[table[x] for x in table.fields]
        )
        r = result.validate()
        if r:
            message = 'The form has errors'
            i18n_errors = {}
            for x in result.errors:
                tran = self.T(result.errors[x])
                i18n_errors[x] = tran
            self.set_status(400)
            return self.write({
                'status': 'Bad Request',
                'code': 400,
                'message': message,
                'errors': result.errors,
                'i18n': {
                    'message': self.T(message),
                    'errors': i18n_errors
                }
            })
        else:
            q_role = db(db.auth_group.id == id_role).select().first()
            result.update(table, q_role.id)
            self.set_status(200)
            return self.write({
                'status': 'OK',
                'code': 200,
                'message': 'Group was successfully changed',
                'auth_group': {
                    'id': str(id_role),
                    'role': E(q_role.role),
                    'grade': q_role.grade,
                    'description': E(q_role.description)
                },
                'i18n': {
                    'message': self.T('Group was successfully changed'),
                    'auth_role': {'role': self.T(q_role.role)}
                }
            })


class Impersonate(web.RequestHandler):
    """
        url: '/api/admin/impersonate/<id_role>'
    """

    def initialize(self, app_name, projectConfig, DALDatabase, i18nTranslator=None, logger_api=None):
        self.app_name = app_name
        self.projectConfig = projectConfig
        self.DALDatabase = DALDatabase
        self.i18nTranslator = i18nTranslator
        if logger_api:
            self.logger_api = logger_api
        if i18nTranslator:
            self.T = i18nTranslator.T
        self.set_header("Access-Control-Allow-Origin", "*")
        self.set_header(
            "Access-Control-Allow-Headers",
            "".join([
                "phanterpwa-language,",
                "phanterpwa-application,",
                "phanterpwa-application-version,",
                "phanterpwa-client-token,",
                "phanterpwa-authorization,",
                "cache-control"
            ])
        )
        self.set_header('Access-Control-Allow-Methods', 'GET, OPTIONS, POST')
        if self.request.headers.get("phanterpwa-language"):
            self.phanterpwa_language = self.request.headers.get("phanterpwa-language")
        else:
            self.phanterpwa_language = browser_language(self.request.headers.get("Accept-Language"))
        if self.i18nTranslator:
            self.i18nTranslator.direct_translation = self.phanterpwa_language
        self.phanterpwa_user_agent = str(self.request.headers.get('User-Agent'))
        self.phanterpwa_remote_ip = self.request.headers.get("X-Real-IP") or \
            self.request.headers.get("X-Forwarded-For") or \
            self.request.remote_ip

    def check_origin(self, origin):
        return True

    def _request_summary(self) -> str:
        client_ip = self.request.headers.get('X-Real-IP') or\
            self.request.headers.get('X-Forwarded-For', '').split(',')[0].strip() or\
            self.request.remote_ip
        summary = "{0} {1} ({2})".format(
            self.request.method,
            self.request.uri,
            client_ip,
        )
        if hasattr(self, "phanterpwa_current_user") and self.phanterpwa_current_user is not None:
            summary = "{0} {1} ({2} - {3})".format(
                self.request.method,
                self.request.uri,
                client_ip,
                self.phanterpwa_current_user.email
            )
        return summary

    def options(self, *args):
        self.set_status(200)
        self.write({"status": "OK"})

    @requires_authentication(roles_name="root")
    def get(self, *args, **kargs):
        id_user = args[0]
        db = self.DALDatabase
        q_user = db(db.auth_user.id == id_user).select()
        if q_user:
            email = q_user.first().email
            result = arbritary_login(
                self.app_name,
                self.projectConfig,
                db,
                email,
                self.phanterpwa_user_agent,
                self.request.remote_ip,
                self.phanterpwa_client_token
            )
            if result:
                self.set_status(200)
                self.write(result)


class RequestAccount(web.RequestHandler):
    """
        url: '/api/admin/request-password/'
    """

    def initialize(self, app_name, projectConfig, DALDatabase, Translator_email, i18nTranslator=None, logger_api=None):
        self.app_name = app_name
        self.projectConfig = projectConfig
        self.DALDatabase = DALDatabase
        self.i18nTranslator = i18nTranslator
        self.Translator_email = Translator_email
        if logger_api:
            self.logger_api = logger_api
        if i18nTranslator:
            self.T = i18nTranslator.T
        self.set_header("Access-Control-Allow-Origin", "*")
        self.set_header(
            "Access-Control-Allow-Headers",
            "".join([
                "phanterpwa-language,",
                "phanterpwa-application,",
                "phanterpwa-application-version,",
                "phanterpwa-client-token,",
                "phanterpwa-authorization,",
                "cache-control"
            ])
        )
        self.set_header('Access-Control-Allow-Methods', 'GET, OPTIONS, POST')
        if self.request.headers.get("phanterpwa-language"):
            self.phanterpwa_language = self.request.headers.get("phanterpwa-language")
        else:
            self.phanterpwa_language = browser_language(self.request.headers.get("Accept-Language"))
        if self.i18nTranslator:
            self.i18nTranslator.direct_translation = self.phanterpwa_language
        self.phanterpwa_user_agent = str(self.request.headers.get('User-Agent'))
        self.phanterpwa_remote_ip = self.request.headers.get("X-Real-IP") or \
            self.request.headers.get("X-Forwarded-For") or \
            self.request.remote_ip

    def check_origin(self, origin):
        return True

    def _request_summary(self) -> str:
        client_ip = self.request.headers.get('X-Real-IP') or\
            self.request.headers.get('X-Forwarded-For', '').split(',')[0].strip() or\
            self.request.remote_ip
        summary = "{0} {1} ({2})".format(
            self.request.method,
            self.request.uri,
            client_ip,
        )
        if hasattr(self, "phanterpwa_current_user") and self.phanterpwa_current_user is not None:
            summary = "{0} {1} ({2} - {3})".format(
                self.request.method,
                self.request.uri,
                client_ip,
                self.phanterpwa_current_user.email
            )
        return summary

    def get(self, *args, **kargs):
        self.set_status(200)
        self.write({"status": "OK"})

    @requires_authentication(roles_name="root")
    @check_private_csrf_token(form_identify=["phanterpwa-form-auth_user"])
    def post(self, *args, **kargs):
        dict_arguments = {k: self.request.arguments.get(k)[0].decode('utf-8') for k in self.request.arguments}
        email = dict_arguments['email']
        result = FieldsDALValidateDictArgs(
            dict_arguments,
            Field(
                'email',
                'string',
                requires=IS_EMAIL(
                    email, error_message="The email isn't valid.")),
        )
        r = result.validate()
        if r:
            message = 'The form has errors'

            i18n_errors = {}
            for x in result.errors:
                tran = self.T(result.errors[x])
                i18n_errors[x] = tran
            self.set_status(400)
            return self.write({
                'status': 'Bad Request',
                'code': 400,
                'message': message,
                'errors': result.errors,
                'i18n': {
                    'message': self.T(message),
                    'errors': i18n_errors
                }
            })
        else:
            q_user = self.DALDatabase(
                self.DALDatabase.auth_user.email == email
            ).select().first()
            if not q_user:
                self.set_status(400)
                return self.write({
                    'status': 'Bad Request',
                    'code': 400,
                    'message': 'The user was not found',
                    'i18n': {'message': self.T('The user was not found')}
                })
            else:
                new_password = temporary_password(chars="abcdefghijmnpqrstuvxzykABCDEFGHJLMNPQRSTUVXZYK123456789")
                self.Translator_email.direct_translation = self.phanterpwa_language
                keys_formatter = dict(
                    app_name=self.projectConfig['PROJECT']['name'],
                    user_name="{0} {1}".format(q_user.first_name, q_user.last_name),
                    password=new_password,
                    time_expires=humanize_seconds(
                        self.projectConfig['BACKEND'][self.app_name]['default_time_temporary_password_expire'],
                        self.Translator_email
                    ),
                    copyright=interpolate(self.projectConfig['CONTENT_EMAILS']['copyright'], {'now': datetime.now().year}),
                    link_to_your_page=self.projectConfig['CONTENT_EMAILS']['link_to_your_site']
                )
                email_password.text.formatter(keys_formatter)
                text_email = email_password.text.html(
                    minify=True,
                    translate=True,
                    formatter=keys_formatter,
                    i18nInstance=self.Translator_email,
                    dictionary=self.phanterpwa_language,
                    do_not_translate=["\n", " ", "\n\n", "&nbsp;"],
                    escape_string=False
                )

                html_email = email_password.html.html(
                    minify=True,
                    translate=True,
                    formatter=keys_formatter,
                    i18nInstance=self.Translator_email,
                    dictionary=self.phanterpwa_language,
                    do_not_translate=["\n", " ", "\n\n", "&nbsp;"],
                    escape_string=False
                )

                e_mail = MailSender(
                    self.projectConfig['EMAIL']['default_sender'],
                    self.projectConfig['EMAIL']['password'],
                    email,
                    subject="Temporary Password Recovery",
                    text_message=text_email,
                    html_message=html_email,
                    server=self.projectConfig['EMAIL']['server'],
                    port=self.projectConfig['EMAIL']['port'],
                    use_tls=self.projectConfig['EMAIL']['use_tls'],
                    use_ssl=self.projectConfig['EMAIL']['use_ssl']
                )
                result = ""
                try:
                    if self.projectConfig["PROJECT"]["debug"]:
                        self.logger_api.warning("TEMPORARY PASSWORD: {0}".format(new_password))
                    else:
                        self.logger_api.warning("Email from '{0}' to '{1}' -> Temporary Password Recovery: {2}".format(
                            self.projectConfig['EMAIL']['default_sender'],
                            email,
                            new_password
                        ))
                        e_mail.send()
                except Exception as e:
                    result = "Email from '{0}' to '{1}' don't send! -> Error: {2} -> password: {3}".format(
                        self.projectConfig['EMAIL']['default_sender'], email, e, new_password)
                    self.logger_api.error(result, exc_info=True)
                    message = "There was an error trying to send the email."
                    message_i18n = self.T("There was an error trying to send the email.")
                    self.set_status(400)
                    return self.write({
                        'status': 'Bad Request',
                        'code': 400,
                        'message': message,
                        'i18n': {'message': message_i18n}
                    })
                else:
                    pass_hash = pbkdf2_sha512.hash("password{0}{1}".format(
                        new_password, self.projectConfig['BACKEND'][self.app_name]['secret_key']))
                    q_user.update_record(
                        temporary_password_hash=pass_hash,
                        temporary_password_expire=datetime.now() +
                            timedelta(seconds=self.projectConfig['BACKEND'][self.app_name]['default_time_temporary_password_expire']),
                        timeout_to_resend_temporary_password_mail=datetime.now() +
                            timedelta(seconds=self.projectConfig['BACKEND'][self.app_name]['timeout_to_resend_temporary_password_mail'])
                    )
                    self.DALDatabase.commit()
                    message = 'An email has been sent instructing how to proceed to recover the account.'
                    self.set_status(200)
                    return self.write({
                        'status': 'OK',
                        'code': 200,
                        'temporary_password': new_password,
                        'email': email,
                        'message': message,
                        'i18n': {
                            'message': self.T(message)
                        }
                    })

    def options(self, *args):
        self.set_status(200)
        self.write({"status": "OK"})


class ChangePassword(web.RequestHandler):
    """
        url: '/api/auth/change-password'
    """

    def initialize(self, app_name, projectConfig, DALDatabase, i18nTranslator=None, logger_api=None):
        self.app_name = app_name
        self.projectConfig = projectConfig
        self.DALDatabase = DALDatabase
        self.i18nTranslator = i18nTranslator
        if logger_api:
            self.logger_api = logger_api
        if i18nTranslator:
            self.T = i18nTranslator.T
        self.set_header("Access-Control-Allow-Origin", "*")
        self.set_header(
            "Access-Control-Allow-Headers",
            "".join([
                "phanterpwa-language,",
                "phanterpwa-application,",
                "phanterpwa-application-version,",
                "phanterpwa-client-token,",
                "phanterpwa-authorization,",
                "cache-control"
            ])
        )
        self.set_header('Access-Control-Allow-Methods', 'GET, OPTIONS')
        if self.request.headers.get("phanterpwa-language"):
            self.phanterpwa_language = self.request.headers.get("phanterpwa-language")
        else:
            self.phanterpwa_language = browser_language(self.request.headers.get("Accept-Language"))
        if self.i18nTranslator:
            self.i18nTranslator.direct_translation = self.phanterpwa_language
        self.phanterpwa_user_agent = str(self.request.headers.get('User-Agent'))
        self.phanterpwa_remote_ip = self.request.headers.get("X-Real-IP") or \
            self.request.headers.get("X-Forwarded-For") or \
            self.request.remote_ip

    def check_origin(self, origin):
        return True

    def _request_summary(self) -> str:
        client_ip = self.request.headers.get('X-Real-IP') or\
            self.request.headers.get('X-Forwarded-For', '').split(',')[0].strip() or\
            self.request.remote_ip
        summary = "{0} {1} ({2})".format(
            self.request.method,
            self.request.uri,
            client_ip,
        )
        if hasattr(self, "phanterpwa_current_user") and self.phanterpwa_current_user is not None:
            summary = "{0} {1} ({2} - {3})".format(
                self.request.method,
                self.request.uri,
                client_ip,
                self.phanterpwa_current_user.email
            )
        return summary

    @requires_authentication(roles_name="root")
    @check_private_csrf_token(form_identify=["phanterpwa-form-auth_user"])
    def post(self):
        dict_arguments = {k: self.request.arguments.get(k)[0].decode('utf-8') for k in self.request.arguments}

        email = dict_arguments['email']
        result = FieldsDALValidateDictArgs(
            dict_arguments,
            Field(
                'email',
                'string',
                requires=IS_EMAIL(
                    email, error_message="The email isn't valid.")),
        )
        r = result.validate()
        if r:
            message = 'The form has errors'

            i18n_errors = {}
            for x in result.errors:
                tran = self.T(result.errors[x])
                i18n_errors[x] = tran
            self.set_status(400)
            return self.write({
                'status': 'Bad Request',
                'code': 400,
                'message': message,
                'errors': result.errors,
                'i18n': {
                    'message': self.T(message),
                    'errors': i18n_errors
                }
            })
        else:
            q_user = self.DALDatabase(
                self.DALDatabase.auth_user.email == email
            ).select().first()
            if not q_user:
                self.set_status(400)
                return self.write({
                    'status': 'Bad Request',
                    'code': 400,
                    'message': 'The user was not found',
                    'i18n': {'message': self.T('The user was not found')}
                })
            else:
                password = dict_arguments.get('password', None)
                if password:
                    pass_hash = pbkdf2_sha512.hash("password{0}{1}".format(
                        password, self.projectConfig['BACKEND'][self.app_name]['secret_key']))
                    q_user.update_record(
                        login_attempts=0,
                        password_hash=pass_hash
                    )
                    self.DALDatabase.commit()
                    self.set_status(200)
                    return self.write({
                        'status': 'OK',
                        'code': 200,
                        'message': 'Password changed!',
                        'i18n': {
                            'message': self.T('Password changed!'),
                        }
                    })
                else:
                    self.set_status(400)
                    return self.write({
                        'status': 'Bad Request',
                        'code': 400,
                    })

    def options(self, *args):
        self.set_status(200)
        self.write({"status": "OK"})



import phanterpwa.frontend.gatehandler as gatehandler
import phanterpwa.frontend.helpers as helpers
import phanterpwa.frontend.forms as forms
import phanterpwa.frontend.components.widgets as widgets
import phanterpwa.frontend.components.left_bar as left_bar
from org.transcrypt.stubs.browser import __pragma__

__pragma__('alias', "jQuery", "$")
__pragma__('skip')
jQuery = sessionStorage = JSON = M = js_undefined = window = setTimeout = \
    __new__ = FormData = console = localStorage = document = this = 0
__pragma__('noskip')

DIV = helpers.XmlConstructor.tagger("div")
I = helpers.XmlConstructor.tagger("i")
H1 = helpers.XmlConstructor.tagger("h1")
H2 = helpers.XmlConstructor.tagger("h2")
H3 = helpers.XmlConstructor.tagger("h3")
STRONG = helpers.XmlConstructor.tagger("strong")
SPAN = helpers.XmlConstructor.tagger("span")
A = helpers.XmlConstructor.tagger("a")
UL = helpers.XmlConstructor.tagger("ul")
LI = helpers.XmlConstructor.tagger("li")
FORM = helpers.XmlConstructor.tagger("form")
LABEL = helpers.XmlConstructor.tagger("label")
HR = helpers.XmlConstructor.tagger("hr", True)
XSECTION = helpers.XSECTION
I18N = helpers.I18N
CONCATENATE = helpers.CONCATENATE
XTABLE = widgets.Table
XML = helpers.XML
XTRD = widgets.TableData
XTRH = widgets.TableHead
XFOOTER = widgets.TableFooterPagination


__pragma__('kwargs')


class Index(gatehandler.Handler):
    def __init__(self, request):
        gatehandler.Handler.__init__(self, request)
        self.projects_data = localStorage.getItem("admin_projects_data")

    def initialize(self):
        html = CONCATENATE(
            DIV(
                DIV(
                    DIV(
                        DIV("PROJECT", _class="phanterpwa-breadcrumb"),
                        DIV(self.request.get_arg(0), _class="phanterpwa-breadcrumb"),
                        _class="phanterpwa-breadcrumb-wrapper"
                    ),
                    _class="p-container"),
                _class='title_page_container card'
            ),
            DIV(
                DIV(
                    DIV(
                        DIV(
                            widgets.Preloaders("preload_android"),
                            _style="width:100%; text-align: center; padding-top: 100px; padding-bottom: 100px;"
                        ),
                        _id="applications_container"),
                    _class="card p-row e-padding_10"
                ),
                _class="phanterpwa-container p-container"
            )
        )
        html.html_to("#main-container")
        BackButton = left_bar.LeftBarButton(
            "back_developer",
            "Voltar",
            I(_class="fas fa-arrow-circle-left"),
            **{
                "_phanterpwa-way": "developer",
                "position": "top",
                "ways": [lambda: window.PhanterPWA.get_current_way().startswith("project")]
            }
        )
        window.PhanterPWA.Components['left_bar'].add_button(BackButton)
        if self.request.get_arg(1) == "view":

            ConfigProject = left_bar.LeftBarButton(
                "config_project",
                "Configurar Projeto e Api",
                I(_class="fas fa-tools"),
                **{
                    "_phanterpwa-way": "project/{0}/config".format(self.request.get_arg(0)),
                    "position": "top",
                    "ways": [lambda: window.PhanterPWA.get_current_way().startswith("project")]
                }
            )
            window.PhanterPWA.Components['left_bar'].add_button(ConfigProject)
            window.PhanterPWA.GET(**{
                'url_args': ['api', 'projects', self.request.get_arg(0)],
                'onComplete': self._after_get_applications
            })
        elif self.request.get_arg(1) == "config":
            window.PhanterPWA.GET(**{
                'url_args': ['api', 'config', self.request.get_arg(0)],
                'onComplete': self._after_get_config
            })

    def _binds(self):
        jQuery("#phanterpwa-component-left_bar-menu_button-change_projects_folder").off(
            "click.change_projects_folder"
        ).on(
            "click.change_projects_folder",
            self.change_projects_folder,
        )

    def reload(self):
        self._binds()

    def change_projects_folder(self):
        admin_authorization = localStorage.getItem("admin_authorization")
        window.PhanterPWA.WS.onMessage = lambda ev: self._after_get_projects_folder(ev)
        window.PhanterPWA.WS.send({
            "authorization": admin_authorization, "command": "change_project_folder"
        })

    def _after_get_projects_folder(self, evt):
        str_json = JSON.parse(evt.data)
        if str_json.status == 200:
            window.PhanterPWA.open_way("developer")

    def _bind_open_api(self, app_name, url):

        jQuery("#phanterpwa-component-left_bar-menu_button-view_api").off(
            "click.open_href_play"
        ).on(
            "click.open_href_play",
            lambda: window.open(url, "_blank")
        )

    def _xml_projects_list(self, json):
        table = XTABLE(
            "applications-table",
            XTRH(
                "applications-table-head",
                *["Application Name", "Build Folder"],
                DIV(
                    I(_class="fas fa-plus"),
                    **{
                        "_phanterpwa-way": "application/new",
                        "_class": "icon_button wave_on_click"
                    }
                )
            )
        )
        apps_dict = dict(json.config.APPS)
        for x in apps_dict.keys():
            table.append(
                XTRD(
                    "applications-table-data-{0}".format(x),
                    x,
                    apps_dict[x]["build_folder"],
                    widgets.MenuBox(
                        "drop_2_{0}".format(x),
                        DIV(I(_class="fas fa-ellipsis-v"), _class="icon_button wave_on_click"),
                        custom_menu=UL(
                            LI("Compile", **{
                                "_id": "btn_compile_app_project_{0}".format(x),
                                "_class": "btn_compile_app_project",
                                "_data-path": json.config['PROJECT']['path'],
                                "_data-app": x
                            }),
                            LI("View", **{
                                "_class": "botao_editar_role",
                                "_phanterpwa-way": "application/{0}/view".format(x)
                            }),
                            LI("Delete", **{
                                "_class": "botao_editar_role",
                                "_phanterpwa-way": "application/{0}/delete".format(x)
                            }),
                            **{
                                "data-menubox": "drop_2_{0}".format(x),
                                "_class": 'dropdown-content'
                            },
                        ),
                        onOpen=lambda: window.PhanterPWA.flash("foi clicado!")
                    )
                )
            )

        url = json.config.BACKEND.api.http_address

        ShowApi = left_bar.LeftBarButton(
            "view_api",
            "Open Api",
            I(_class="fas fa-globe"),
            **{
                "position": "top",
                "ways": [lambda: True if json.running is True and
                    window.PhanterPWA.get_current_way().startswith("project/{0}".format(self.request.get_arg(0))) else False],
                "onStart": lambda: self._bind_open_api(self.request.get_arg(0), url)
            }
        )
        window.PhanterPWA.Components['left_bar'].add_button(ShowApi)

        html = DIV(
            XSECTION(
                LABEL("Summary"),
                DIV(
                    DIV(
                        DIV(
                            DIV(
                                STRONG("PROJECT PATH"),
                                SPAN(json.config['PROJECT']['path']),
                                _class="e-tagger-wrapper"
                            ),
                            _class="p-col w1p100"
                        ),
                        _class="p-row"
                    ),
                    DIV(
                        DIV(
                            DIV(
                                STRONG("IDENTIFIER NAME"),
                                SPAN(json.config['PROJECT']['name']),
                                _class="e-tagger-wrapper"
                            ),
                            _class="p-col w1p100"
                        ),
                        _class="p-row"
                    ),
                    DIV(
                        DIV(
                            DIV(
                                STRONG("TITLE"),
                                SPAN(json.config['PROJECT']['title']),
                                _class="e-tagger-wrapper"
                            ),
                            _class="p-col w1p100"
                        ),
                        _class="p-row"
                    ),
                    DIV(
                        DIV(
                            DIV(
                                STRONG("AUTHOR"),
                                SPAN(json.config['PROJECT']['author']),
                                _class="e-tagger-wrapper"
                            ),
                            _class="p-col w1p100"
                        ),
                        _class="p-row"
                    ),
                    DIV(
                        DIV(
                            DIV(
                                STRONG("VERSION"),
                                SPAN(json.config['PROJECT']['version']),
                                _class="e-tagger-wrapper"
                            ),
                            _class="p-col w1p100 w3p50"
                        ),
                        DIV(
                            DIV(
                                STRONG("COMPILATION"),
                                SPAN(json.config['PROJECT']['compilation']),
                                _class="e-tagger-wrapper"
                            ),
                            _class="p-col w1p100 w3p50"
                        ),
                        _class="p-row"
                    ),
                    DIV(
                        DIV(
                            DIV(
                                STRONG("DEBUG"),
                                SPAN(json.config['PROJECT']['debug']),
                                _class="e-tagger-wrapper"
                            ),
                            _class="p-col w1p100 w3p50"
                        ),
                        DIV(
                            DIV(
                                STRONG("STATUS"),
                                SPAN("Running" if json.running else "Stopped", _style="color: {0};".format(
                                    "green" if json.running else "red")),
                                _class="e-tagger-wrapper"
                            ),
                            _class="p-col w1p100 w3p50"
                        ),
                        _class="p-row"
                    ),
                    _class="e-padding_20"
                )
            ),
            table
        )
        html.html_to("#applications_container")
        self.widgets_initialize()

    def _xml_config_project(self, json):
        project_config = dict(json.project_config['PROJECT'])
        email_config = dict(json.project_config['EMAIL'])
        email_contents_config = dict(json.project_config['CONTENT_EMAILS'])
        api_config = dict(json.api_config['API'])
        project_default = [
            "title",
            "version",
            "author",
            "debug",
            "packaged"
        ]
        email_secret_default = [
            "password"
        ]
        email_default = [
            "server",
            "username",
            "default_sender",
            "port",
            "use_tls",
            "use_ssl"
        ]
        content_email_default = [
            "copyright",
            "link_to_your_site"
        ]
        html = FORM(H2("PROJECT"), HR(), **{"_phanterpwa-form": "config_project"})
        for x in project_config.keys():
            if x == "debug" or x == "packaged":
                html.append(widgets.CheckBox("project_{0}".format(x), **{
                    "value": project_config[x],
                    "name": x,
                    "label": x.replace("_", " ").capitalize(),
                    "form": "config_project"
                }))
            else:
                html.append(widgets.Input("project_{0}".format(x), **{
                    "value": project_config[x],
                    "name": x,
                    "label": x.replace("_", " ").capitalize(),
                    "form": "config_project"
                }))
        html.append(CONCATENATE(H2("EMAIL"), HR()))
        for x in email_config.keys():
            if x == "use_tls" or x == "use_ssl":
                html.append(widgets.CheckBox("emai_{0}".format(x), **{
                    "value": email_config[x],
                    "name": x,
                    "label": x.replace("_", " ").capitalize(),
                    "form": "config_project"
                }))
            else:
                html.append(widgets.Input("emai_{0}".format(x), **{
                    "value": email_config[x],
                    "name": x,
                    "label": x.replace("_", " ").capitalize(),
                    "form": "config_project"
                }))
        html.append(CONCATENATE(H2("CONTENT EMAILS"), HR()))
        for x in email_contents_config.keys():
            html.append(widgets.Input("content_emails_{0}".format(x), **{
                "value": email_contents_config[x],
                "name": x,
                "label": x.replace("_", " ").capitalize(),
                "form": "config_project"
            }))
        html.append(CONCATENATE(H2("API"), HR()))
        for x in api_config.keys():
            if x == "secret_key" or x == "url_secret_key":
                html.append(widgets.Input("api_{0}".format(x), **{
                    "value": api_config[x],
                    "name": x,
                    "label": x.replace("_", " ").capitalize(),
                    "form": "config_project",
                    "icon": I(_class="fab fa-sistrix"),
                    "icon_on_click": lambda: window.PhanterPWA.flash(**{"html": I18N("Fui Clicado!")})
                }))
            else:
                html.append(widgets.Input("api_{0}".format(x), **{
                    "value": api_config[x],
                    "name": x,
                    "label": x.replace("_", " ").capitalize(),
                    "form": "config_project"
                }))
        html.append(forms.SubmitButton("config_project", "Save Config"))
        html.html_to("#applications_container")

    def _after_get_applications(self, data, ajax_status):
        if ajax_status == "success":
            json = data.responseJSON
            if data.status == 200:
                localStorage.setItem("admin_projects_data", JSON.stringify(json))
                self.admin_projects_data = json
                window.PhanterPWA.flash(**{'html': json.i18n.message})

                self._xml_projects_list(json)

            elif data.status == 202:
                window.PhanterPWA.flash(**{'html': json.i18n.message})
            localStorage.setItem("admin_authorization", json.authorization)
            self.admin_authorization = json.authorization
            self._binds()
        else:
            window.PhanterPWA.flash("Problem on server: {0}".format(str(data.status)))

    def _after_get_config(self, data, ajax_status):
        if ajax_status == "success":
            json = data.responseJSON
            if data.status == 200:
                window.PhanterPWA.flash(**{'html': json.i18n.message})
                self._xml_config_project(json)

        else:
            window.PhanterPWA.flash("Problem on server: {0}".format(str(data.status)))


__pragma__('nokwargs')



import phanterpwa.frontend.helpers as helpers
import phanterpwa.frontend.components.widgets as widgets
from org.transcrypt.stubs.browser import __pragma__
__pragma__('alias', "jQuery", "$")
__pragma__('skip')
jQuery = sessionStorage = JSON = M = js_undefined = window =\
    __new__ = FormData = console = localStorage = document = this = CodeMirror = 0
__pragma__('noskip')


DIV = helpers.XmlConstructor.tagger("div")


__pragma__('kwargs')


class CodeMirrorHelper(widgets.Widget):
    def __init__(self, identifier, **parameters):
        self.code_mirror_parameters = dict()
        for x in parameters.keys():
            if not x.startswith("_"):
                self.code_mirror_parameters[x] = parameters[x]
        parameters["_id"] = identifier
        if "_class" in parameters:
            parameters["_class"] = "{0}{1}".format(
                parameters["_class"].strip(),
                " phanterpwa-plugin-codemirrorhelper"
            )
        else:
            parameters["_class"] = "phanterpwa-plugin-codemirrorhelper"
        self._codemirror_target = window.PhanterPWA.get_id(identifier)
        html = DIV(_id=self._codemirror_target, _class="source_code-codemirror-wrapper")
        widgets.Widget.__init__(self, identifier, html, **parameters)

    def reload(self):
        self.start()

    def start(self):
        self.CodeMirror = CodeMirror(jQuery("#{0}".format(self._codemirror_target)).html("")[0],
            self.code_mirror_parameters)


__pragma__('nokwargs')



"""
Title: helpers

Author: PhanterJR<junior.conex@gmail.com>

License: MIT

Coding: utf-8

In this module are gathered all the TAGs currently used for the creation of HTML5 pages.
TAG objects inherited the functionality of their parent class, the XmlConstructor

You can import all with.

    >>> from phanterpwa.helpers import *

Or just what you want.

    >>> from phanterpwa.helpers import (HTML, HEAD, BODY, DIV, SPAN)
    >>> HTML(HEAD(), BODY(DIV(SPAN())))
    <!DOCTYPE html><head><head><body><div><span></span></div></boby></html>

The complete list can be obtained with.

    >>> from phanterpwa.helpers import ALL_TAGS
    >>> print(ALL_TAGS)
    ['CONCATENATE', 'XML', 'XCOMMENT', 'AREA', 'BASE', 'COL', 'EMBED', 'HR', 'IMG', 'INPUT', 'LINK', 'META', 'PARAM', 'SOURCE', 'TRACK', 'WBR', 'BR', 'A', 'ABBR', 'ADDRESS', 'ARTICLE', 'ASIDE', 'AUDIO', 'B', 'BDI', 'BDO', 'BLOCKQUOTE', 'BODY', 'BUTTON', 'CANVAS', 'CAPTION', 'CITE', 'CODE', 'COLGROUP', 'DATA', 'DATALIST', 'DD', 'DEL', 'DETAILS', 'DFN', 'DIALOG', 'DIV', 'DL', 'DT', 'EM', 'FIELDSET', 'FIGCAPTION', 'FIGURE', 'FOOTER', 'FORM', 'H1', 'H2', 'H3', 'H4', 'H5', 'H6', 'HEAD', 'HEADER', 'HTML', 'I', 'IFRAME', 'INS', 'KBD', 'LABEL', 'LEGEND', 'LI', 'MAIN', 'MAP', 'MARK', 'METER', 'NAV', 'NOSCRIPT', 'OBJECT', 'OL', 'OPTGROUP', 'OPTION', 'OUTPUT', 'P', 'PICTURE', 'PRE', 'PROGRESS', 'Q', 'RP', 'RT', 'RUBY', 'S', 'SAMP', 'SCRIPT', 'SECTION', 'SELECT', 'SMALL', 'SPAN', 'STRONG', 'STYLE', 'SUB', 'SUMMARY', 'SUP', 'SVG', 'TABLE', 'TBODY', 'TD', 'TEMPLATE', 'TEXTAREA', 'TFOOT', 'TH', 'THEAD', 'TIME', 'TITLE', 'TR', 'U', 'UL', 'VAR', 'VIDEO']

There is also a separate list by categories: NORMAL_TAGS, VOID_TAGS,
and SPECIAL_TAGS.
"""

from .xmlconstructor import XmlConstructor
from .third_parties.xss import XssCleaner


SPECIAL_TAGS = [
    'CONCATENATE',
    'XML',
    'XCOMMENT'
]
VOID_TAGS = [
    'AREA',
    'BASE',
    'COL',
    'EMBED',
    'HR',
    'IMG',
    'INPUT',
    'LINK',
    'META',
    'PARAM',
    'SOURCE',
    'TRACK',
    'WBR',
    'BR'
]
NORMAL_TAGS = [
    'A',
    'ABBR',
    'ADDRESS',
    'ARTICLE',
    'ASIDE',
    'AUDIO',
    'B',
    'BDI',
    'BDO',
    'BLOCKQUOTE',
    'BODY',
    'BUTTON',
    'CANVAS',
    'CAPTION',
    'CITE',
    'CODE',
    'COLGROUP',
    'DATA',
    'DATALIST',
    'DD',
    'DEL',
    'DETAILS',
    'DFN',
    'DIALOG',
    'DIV',
    'DL',
    'DT',
    'EM',
    'FIELDSET',
    'FIGCAPTION',
    'FIGURE',
    'FOOTER',
    'FORM',
    'H1',
    'H2',
    'H3',
    'H4',
    'H5',
    'H6',
    'HEAD',
    'HEADER',
    'HTML',
    'I',
    'IFRAME',
    'INS',
    'KBD',
    'LABEL',
    'LEGEND',
    'LI',
    'MAIN',
    'MAP',
    'MARK',
    'METER',
    'NAV',
    'NOSCRIPT',
    'OBJECT',
    'OL',
    'OPTGROUP',
    'OPTION',
    'OUTPUT',
    'P',
    'PICTURE',
    'PRE',
    'PROGRESS',
    'Q',
    'RP',
    'RT',
    'RUBY',
    'S',
    'SAMP',
    'SCRIPT',
    'SECTION',
    'SELECT',
    'SMALL',
    'SPAN',
    'STRONG',
    'STYLE',
    'SUB',
    'SUMMARY',
    'SUP',
    'SVG',
    'TABLE',
    'TBODY',
    'TD',
    'TEMPLATE',
    'TEXTAREA',
    'TFOOT',
    'TH',
    'THEAD',
    'TIME',
    'TITLE',
    'TR',
    'U',
    'UL',
    'VAR',
    'VIDEO',
]
ALL_TAGS = __all__ = SPECIAL_TAGS + VOID_TAGS + NORMAL_TAGS


class CONCATENATE(XmlConstructor):
    """With CONCATENATE it is possible to concatenate several objects to create the html.

    Example:
        >>> from phanterpwa.helpers import DIV, CONCATENATE
        >>> my_instance = CONCATENATE(
        ...     DIV("This is a div"),
        ...     "this is a string",
        ...     DIV("this is other div")
        ... )
        >>> print(my_instance)
        <div>This is a div</div>this is a string<div>this is other div</div>
    """

    def __init__(self, *content):
        XmlConstructor.__init__(self, "", False, *content)
        self.alternative_tag = "concatenate"


class XML(XmlConstructor, XssCleaner):
    """With XML it is possible to apply an optional sanitization, it is disabled by default, so that every string will
    be treated as html.

    Example:
        >>> from phanterpwa.helpers import SPAN, DIV, XML
        >>> instanceDIV = DIV("<span>escaped</span>")
        >>> print(instanceDIV)
        <div>&lt;span&gt;escaped&lt;/span&gt;</div>
        >>> instanceDIV_with_XML = DIV(XML("<span>not escaped</span>"))
        >>> print(instanceDIV_with_XML)
        <div><span>not escaped</span></div>
        >>> instanceSPAN = SPAN("<evil>I will destroy you.</evil>")
        >>> print(instanceSPAN)
        <span>&lt;evil&gt;I will destroy you.&lt;/evil&gt;</span>
        >>> print(DIV("<other_evil>", XML("<evil>I will destroy you.</evil>"), "</other_evil>"))
        <div>&lt;other_evil&gt;<evil>I will destroy you.</evil>&lt;/other_evil&gt;</div>
    """

    def __init__(self,
            *content,
            sanitize=False,
            permitted_tags=[
                'a',
                'b',
                'blockquote',
                'br/',
                'i',
                'li',
                'ol',
                'ul',
                'p',
                'cite',
                'code',
                'pre',
                'img/',
            ],
            allowed_attributes={
                'a': ['href', 'title'],
                'img': ['src', 'alt'],
                'blockquote': ['type']
            }):
        XmlConstructor.__init__(self, "", False, content)
        XssCleaner.__init__(self)
        self.alternative_tag = "xml"
        self.sanitize = sanitize
        self.permitted_tags = permitted_tags
        self.allowed_attributes = allowed_attributes
        self.strip_disallowed = False
        self.escape_string = False

    def xml(self) -> str:
        """With this method a string is converted to html with an optional sanitization (predefined tags, predefined
        attributes, escape or not strings), by default sanitization is disabled (False).
        """
        if self.minify:
            xml = self._minified(
                close_void=self.close_void,
                i18nInstance=self.i18nInstance,
                dictionary=self.dictionary,
                do_not_translate=self.do_not_translate,
                tag_translation=self.tag_translation
            )
            if self._formatter:
                xml = self.interpolate(xml, self._formatter, self.delimiters)
        else:
            xml = self.humanize()
        if self.sanitize:
            return self.strip(xml)
        return xml


class XCOMMENT(XmlConstructor):
    """With XCOMMENT it is possible to create html comments.

        Example:
            >>> from phanterpwa.helpers import XCOMMENT
            >>> print(XCOMMENT("thit is a comment"))
            <!--this is a comment-->

    """

    def __init__(self, *content):
        XmlConstructor.__init__(self, "", False, *content)
        self.alternative_tag = "xcomment"
        self.escape_string = False
        self.before_xml = "<!--"
        self.after_xml = "-->"

    def _humanized(self, indent_size=2, close_void=False, translate=False):
        return "".join(["\n", " " * (indent_size * (self._indent_level)), self.xml()])


# html5 by default
class HTML(XmlConstructor):
    def __init__(self, *content, **attributes):
        XmlConstructor.__init__(self, "html", False, *content, **attributes)
        self.before_xml = "<!DOCTYPE html>"


# void tags
AREA = XmlConstructor.tagger("area", True)
BASE = XmlConstructor.tagger("base", True)
COL = XmlConstructor.tagger("col", True)
EMBED = XmlConstructor.tagger("embed", True)
HR = XmlConstructor.tagger("hr", True)
IMG = XmlConstructor.tagger("img", True)
INPUT = XmlConstructor.tagger("input", True)
LINK = XmlConstructor.tagger("link", True)
META = XmlConstructor.tagger("meta", True)
PARAM = XmlConstructor.tagger("param", True)
SOURCE = XmlConstructor.tagger("source", True)
TRACK = XmlConstructor.tagger("track", True)
WBR = XmlConstructor.tagger("wbr", True)
BR = XmlConstructor.tagger("br", True)
# normal tags
A = XmlConstructor.tagger("a")
ABBR = XmlConstructor.tagger("abbr")
ADDRESS = XmlConstructor.tagger("address")
ARTICLE = XmlConstructor.tagger("article")
ASIDE = XmlConstructor.tagger("aside")
AUDIO = XmlConstructor.tagger("audio")
B = XmlConstructor.tagger("b")
BDI = XmlConstructor.tagger("bdi")
BDO = XmlConstructor.tagger("bdo")
BLOCKQUOTE = XmlConstructor.tagger("blockquote")
BODY = XmlConstructor.tagger("body")
BUTTON = XmlConstructor.tagger("button")
CANVAS = XmlConstructor.tagger("canvas")
CAPTION = XmlConstructor.tagger("caption")
CITE = XmlConstructor.tagger("cite")
CODE = XmlConstructor.tagger("code")
COLGROUP = XmlConstructor.tagger("colgroup")
DATA = XmlConstructor.tagger("data")
DATALIST = XmlConstructor.tagger("datalist")
DD = XmlConstructor.tagger("dd")
DEL = XmlConstructor.tagger("del")
DETAILS = XmlConstructor.tagger("details")
DFN = XmlConstructor.tagger("dfn")
DIALOG = XmlConstructor.tagger("dialog")
DIV = XmlConstructor.tagger("div")
DL = XmlConstructor.tagger("dl")
DT = XmlConstructor.tagger("dt")
EM = XmlConstructor.tagger("em")
FIELDSET = XmlConstructor.tagger("fieldset")
FIGCAPTION = XmlConstructor.tagger("figcaption")
FIGURE = XmlConstructor.tagger("figure")
FOOTER = XmlConstructor.tagger("footer")
FORM = XmlConstructor.tagger("form")
H1 = XmlConstructor.tagger("h1")
H2 = XmlConstructor.tagger("h2")
H3 = XmlConstructor.tagger("h3")
H4 = XmlConstructor.tagger("h4")
H5 = XmlConstructor.tagger("h5")
H6 = XmlConstructor.tagger("h6")
HEAD = XmlConstructor.tagger("head", never_translate=True)
HEADER = XmlConstructor.tagger("header")
I = XmlConstructor.tagger("i")
IFRAME = XmlConstructor.tagger("iframe")
INS = XmlConstructor.tagger("ins")
KBD = XmlConstructor.tagger("kbd")
LABEL = XmlConstructor.tagger("label")
LEGEND = XmlConstructor.tagger("legend")
LI = XmlConstructor.tagger("li")
MAIN = XmlConstructor.tagger("main")
MAP = XmlConstructor.tagger("map")
MARK = XmlConstructor.tagger("mark")
METER = XmlConstructor.tagger("meter")
NAV = XmlConstructor.tagger("nav")
NOSCRIPT = XmlConstructor.tagger("noscript")
OBJECT = XmlConstructor.tagger("object")
OL = XmlConstructor.tagger("ol")
OPTGROUP = XmlConstructor.tagger("optgroup")
OPTION = XmlConstructor.tagger("option")
OUTPUT = XmlConstructor.tagger("output")
P = XmlConstructor.tagger("p")
PICTURE = XmlConstructor.tagger("picture")
PRE = XmlConstructor.tagger("pre")
PROGRESS = XmlConstructor.tagger("progress")
Q = XmlConstructor.tagger("q")
RP = XmlConstructor.tagger("rp")
RT = XmlConstructor.tagger("rt")
RUBY = XmlConstructor.tagger("ruby")
S = XmlConstructor.tagger("s")
SAMP = XmlConstructor.tagger("samp")
SCRIPT = XmlConstructor.tagger("script", escape_string=False, never_translate=True)
SECTION = XmlConstructor.tagger("section")
SELECT = XmlConstructor.tagger("select")
SMALL = XmlConstructor.tagger("small")
SPAN = XmlConstructor.tagger("span")
STRONG = XmlConstructor.tagger("strong")
STYLE = XmlConstructor.tagger("style", escape_string=False, never_translate=True)
SUB = XmlConstructor.tagger("sub")
SUMMARY = XmlConstructor.tagger("summary")
SUP = XmlConstructor.tagger("sup")
SVG = XmlConstructor.tagger("svg")
TABLE = XmlConstructor.tagger("table")
TBODY = XmlConstructor.tagger("tbody")
TD = XmlConstructor.tagger("td")
TEMPLATE = XmlConstructor.tagger("template")
TEXTAREA = XmlConstructor.tagger("textarea")
TFOOT = XmlConstructor.tagger("tfoot")
TH = XmlConstructor.tagger("th")
THEAD = XmlConstructor.tagger("thead")
TIME = XmlConstructor.tagger("time")
TITLE = XmlConstructor.tagger("title")
TR = XmlConstructor.tagger("tr")
U = XmlConstructor.tagger("u")
UL = XmlConstructor.tagger("ul")
VAR = XmlConstructor.tagger("var")
VIDEO = XmlConstructor.tagger("video")



#?from org.transcrypt.stubs.browser import __pragma__
from phanterpwa.frontend import (
    helpers,
    decorators,
    application
)
# __pragma__ ('ecom')
#?__pragma__('alias', "jQuery", "$")
#?__pragma__('skip')

# it is ignored on transcrypt
window = console = __new__ = Date = URL = sessionStorage = \
    js_undefined = jQuery = this = Object = JSON = 0

#?__pragma__('noskip')

DIV = helpers.XmlConstructor.tagger("div")
P = helpers.XmlConstructor.tagger("P")
H3 = helpers.XmlConstructor.tagger("h3")
H2 = helpers.XmlConstructor.tagger("h2")
IMG = helpers.XmlConstructor.tagger("img", True)
I18N = helpers.I18N
CONCATENATE = helpers.CONCATENATE

#?__pragma__('kwargs')


class RequestHandler():
    def __init__(self, request, **parameters):
        if not isinstance(request, application.WayRequest):
            raise ValueError()
        self.request = request
        self.current_user = window.PhanterPWA.get_auth_user()
        self.on_error = parameters.get("on_error", None)
        self._status_code = 200
        self._reason = "Unknown"
        self._settings = dict()
        self._preview()
        self._initialize()
        self._callbacks_on_error = []
        self._callbacks_on_finish = []

    @property
    def status_code(self):
        return self._status_code

    @status_code.setter
    def status_code(self, value):
        if str(value).isdigit():
            self._status_code = value
        else:
            raise ValueError("The status_code must be integer.")
    
    def set_status(self, status_code: int, reason: str = None) -> None:
        """Sets the status code for your response.

        :arg int status_code: Response status code.
        :arg str reason: Human-readable reason phrase describing the status

        """
        self.status_code = status_code
        if isinstance(reason, str):
            self._reason = reason
        else:
            self._reason = "Unknown"

    def get_status(self) -> int:
        """Returns the status code for your response."""
        return self._status_code

    def _error(self, code=200):
        if code != 200:
            self.status = code
            if callable(self.on_error):
                self.on_error(code, self)
            window.PhanterPWA.open_code_way(code, self, self.Request)

    def _preview(self):
        if window.PhanterPWA.DEBUG:
            console.info("method not used: RequestHandler._preview")
            console.info("call preview method")
        self.preview()

    def _initialize(self):
        if window.PhanterPWA.DEBUG:
            console.info("method not used: RequestHandler._initialize")
            console.info("call initialize method")
        self.initialize()

    def preview(self):
        if window.PhanterPWA.DEBUG:
            console.info("method not used: RequestHandler.preview")

    def initialize(self, *args, **kwargs):
        if window.PhanterPWA.DEBUG:
            console.info("method not used: RequestHandler.initialize")

    def finish(self):
        if self.status == 200:
            window.PhanterPWA.Response = self


class Handler():

    def __init__(self, request, **parameters):
        self._request = request
        self.debug = window.PhanterPWA.DEBUG
        self.config = window.PhanterPWA.CONFIG
        self._authorized_roles = "all"
        self._authorized_users = "all"
        self._back = window.PhanterPWA.get_current_way()
        self._status = 401
        self._on_error = parameters.get("on_error", None)
        self._current_user = window.PhanterPWA.get_auth_user()
        self._initialize()

    @property
    def auth_user(self):
        self._current_user = window.PhanterPWA.get_auth_user()
        return self._current_user    

    @property
    def current_user(self):
        self._current_user = window.PhanterPWA.get_auth_user()
        return self._current_user

    def stringify(self):
        seen = []
        def f(k, v):
            #?is_obj = __pragma__("js", "{}", "typeof v == 'object'")
            is_obj = isinstance(v, object) #__: skip
            if v is not None and is_obj:
                test = v in seen #__: skip
                #?test = __pragma__("js", "{}", "seen.indexOf(v) >= 0")
                if k.startswith("_"):
                    return
                elif test:
                    return
                seen.append(v)
            return v

        return JSON.stringify(self, lambda k, v: f(k, v))

    def _check_authorized_users(self):
        if self._authorized_users == "all":
            return True
        elif self._authorized_users == "login":
            if self.auth_user is not None:
                return True
        elif self._authorized_users == "nologin":
            if self.auth_user is None:
                return True
        elif isinstance(self._authorized_users, list):
            if self.auth_user is not None and int(self.auth_user.id) in self._authorized_users:
                return True
        return False

    def _check_authorized_roles(self):
        if self._authorized_roles == "all":
            return True
        elif isinstance(self._authorized_roles, str) and self.auth_user is not None:
            if self._authorized_roles in self.auth_user.roles:
                return True
        elif isinstance(self._authorized_roles, list):
            if self.auth_user is not None and len(
                    set(self.auth_user.roles).intersection(set(self.autorized_roles))) > 0:
                return True
        return False

    def check_has_authorization(self):
        self.auth_user = window.PhanterPWA.get_auth_user()
        if self._check_authorized_users() and self._check_authorized_roles():
            return True
        elif self.auth_user is not None:
            if self._authorized_users == "nologin":
                return 400
            else:
                return 403
        return 401

    def on_click_back_button(self):
        window.PhanterPWA.open_way(self._back)

    def _on_credentials_fail(self):
        window.PhanterPWA.open_code_way(self._has_authorization, self._request, self)

    def _initialize(self):
        if window.PhanterPWA.DEBUG:
            console.info("method not used: RequestHandler._initialize")
            console.info("call initialize method")
        self.request = self._request
        self.initialize()
        window.PhanterPWA.reload_components()
        self._has_authorization = self.check_has_authorization()
        if self._has_authorization is True:
            window.PhanterPWA.Response = self
            self.set_title()
        else:
            if callable(self._on_error):
                self._on_error(self._has_authorization, self)
            else:
                self._on_credentials_fail()

    def initialize(self):
        if window.PhanterPWA.DEBUG:
            console.info("method not used: Handler.initialize")

    def widgets_initialize(self):
        for x in self.request.widgets.keys():
            pass
            #self.request.widgets[x].start()

    def set_title(self, value=None):
        if value is None:
            title = jQuery("title").text()
            if window.PhanterPWA.CONFIG.PROJECT.title != title:
                jQuery("title").text(window.PhanterPWA.CONFIG.APP.title or window.PhanterPWA.CONFIG.PROJECT.title)
        else:
            jQuery("title").text(value)



class ErrorHandler():
    def __init__(self, request, response=None, reasons=None):
        self._request = request
        self.reasons = reasons
        self.debug = window.PhanterPWA.DEBUG
        self.requires_login = False
        self.autorized_roles = ["all"]
        self.autorized_ids = ["all"]
        self.way_on_back = window.PhanterPWA.get_current_way()
        self.initialize()
        self._credentials = True
        if self._credentials is True:
            window.PhanterPWA.Response = response
            self.start()
        else:
            self.on_credentials_fail()

    def on_click_back_button(self):
        window.PhanterPWA.open_way(self.way_on_back)

    def on_credentials_fail(self):
        if self.debug:
            console.info('method not used: ErrorHandler.on_credentials_fail()')

    def initialize(self):
        if self.debug:
            console.info("method not used: ErrorHandler.initialize()")

    def start(self):
        if self.debug:
            console.info("method not used: ErrorHandler.start()")


html_base = CONCATENATE(
    DIV(
        DIV(
            DIV(
                DIV(I18N("Ops!!!", **{"_pt-br": "Ops!!!"}), _class="phanterpwa-breadcrumb"),
                _class="phanterpwa-breadcrumb-wrapper"
            ),
            _class="phanterpwa-container container"),
        _class='title_page_container card'
    ),
    DIV(
        DIV(
            DIV(
                DIV(
                    DIV(
                        DIV(
                            IMG(
                                _class='image-warnings'
                            ),
                            _class="image-warnings-container"
                        ),
                        DIV(_id='content-warning'),
                        _class='content-warnings'
                    ),
                    _class='warnings-container phanterpwa-card-container'
                ),
                _class="card"
            ),
            _class="new-container"
        ),
        _class="phanterpwa-container container"
    )
)


class Error_401(ErrorHandler):
    def start(self):
        html = CONCATENATE(
            DIV(
                DIV(
                    DIV(
                        DIV(I18N("Authentication required", **{"_pt-br": "Necessário Autenticar-se"}), _class="phanterpwa-breadcrumb"),
                        _class="phanterpwa-breadcrumb-wrapper"
                    ),
                    _class="phanterpwa-container container"),
                _class='title_page_container card'
            ),
            DIV(
                DIV(
                    DIV(
                        DIV(
                            DIV(
                                IMG(
                                    _class='image-warnings',
                                    _src="/static/{0}/images/warning.png".format(window.PhanterPWA.VERSIONING)
                                ),
                            ),
                            _class="image-warnings-container"
                        ),
                        DIV(
                            H3(I18N(
                                "You need authentication to access this feature.",
                                **{
                                    "_pt-br": "Você precisa autenticar-se para tentar acessar este recurso."
                                }
                            )),
                            H3(self.reasons) if self.reasons is not None else "",
                            _id='content-warning',
                            _class='content-warnings'
                        ),
                        DIV(
                            DIV(
                                I18N("Login"),
                                _id="alternative_login_button",
                                _class="btn wave_on_click link"
                            ),
                            _class="button-container"
                        ),
                        _class='warnings-container card phanterpwa-card-container'
                    ),
                    _class="new-container"
                ),
                _class="phanterpwa-container container"
            )
        )
        html.html_to("#main-container")
        jQuery("#alternative_login_button").off("click.alternative_login_button").on(
            "click.alternative_login_button",
            lambda: window.PhanterPWA.Components["auth_user"].modal_login()
        )


class Error_403(ErrorHandler):
    def start(self):
        html = html_base.jquery()
        html.find(".image-warnings").attr(
            "src", "/static/{0}/images/warning.png".format(window.PhanterPWA.VERSIONING))
        html_warning = CONCATENATE(
            H3("ERROR 403 - Forbidden"),
            P(self.reasons) if self.reasons is not None else ""
        )
        html.find("#content-warning").html(html_warning.jquery())
        jQuery("#main-container").html(html)


class Error_404(ErrorHandler):
    def start(self):
        html = html_base.jquery()
        html.find(".image-warnings").attr(
            "src", "/static/{0}/images/warning.png".format(window.PhanterPWA.VERSIONING))
        html_warning = CONCATENATE(
            H3("ERROR 404 - Not Found"),
            P(self.reasons) if self.reasons is not None else ""
        )
        html.find("#content-warning").html(html_warning.jquery())
        jQuery("#main-container").html(html)


class Error_409(ErrorHandler):
    def start(self):
        html = html_base.jquery()
        html.find(".image-warnings").attr(
            "src", "/static/{0}/images/warning.png".format(window.PhanterPWA.VERSIONING))
        html_warning = CONCATENATE(
            H3("ERROR 404 - Not Found"),
            P(self.reasons) if self.reasons is not None else ""
        )
        html.find("#content-warning").html(html_warning.jquery())
        jQuery("#main-container").html(html)

class Error_500(ErrorHandler):
    def start(self):
        html = html_base.jquery()
        html.find(".image-warnings").attr(
            "src", "/static/{0}/images/warning.png".format(window.PhanterPWA.VERSIONING))
        html_warning = CONCATENATE(
            H3("ERROR 500 - Internal Error"),
            P(self.reasons) if self.reasons is not None else ""
        )
        html.find("#content-warning").html(html_warning.jquery())
        jQuery("#main-container").html(html)

class Error_502(ErrorHandler):
    def start(self):
        html = html_base.jquery()
        html.find(".image-warnings").attr(
            "src", "/static/{0}/images/warning.png".format(window.PhanterPWA.VERSIONING))
        html_warning = CONCATENATE(
            H3("ERROR 502 - Bad Gateway"),
            P(self.reasons) if self.reasons is not None else ""
        )
        html.find("#content-warning").html(html_warning.jquery())
        jQuery("#main-container").html(html)



#?__pragma__('nokwargs')
# __pragma__ ('noecom')


from datetime import datetime
from pydal import Field
from pydal.validators import (
    IS_NOT_EMPTY,
    IS_DATETIME,
    IS_IN_DB,
    IS_EMPTY_OR
)


class MessagesTables():
    def __init__(self, DALDatabase):
        self.DALDatabase = DALDatabase
        self.DALDatabase.define_table(
            'internal_messages',
            Field('senders', 'reference auth_user', notnull=True, requires=IS_IN_DB(
                self.DALDatabase, self.DALDatabase.auth_user)),
            Field('subject', 'string', requires=IS_NOT_EMPTY()),
            Field('text_message', 'string', requires=IS_NOT_EMPTY()),
            Field('message_sent', 'boolean', default=False),
            Field('send_on', 'datetime', requires=IS_EMPTY_OR(IS_DATETIME())),
            Field('written_on', 'datetime', default=datetime.now(), requires=IS_EMPTY_OR(IS_DATETIME()))
        )

        self.DALDatabase.define_table(
            'internal_messages_recipients',
            Field('internal_messages', 'reference internal_messages', notnull=True, requires=IS_IN_DB(
                self.DALDatabase, self.DALDatabase.internal_messages)),
            Field('recipients', 'reference auth_user', notnull=True, requires=IS_IN_DB(
                self.DALDatabase, self.DALDatabase.auth_user)),
            Field('message_read', 'boolean', default=False),
        )



import phanterpwa.frontend.helpers as helpers
import phanterpwa.frontend.components.snippets as snippets
from org.transcrypt.stubs.browser import __pragma__


__pragma__('alias', "jQuery", "$")
__pragma__('skip')
jQuery = sessionStorage = JSON = js_undefined = setTimeout = window = this = console =\
    localStorage = 0
__pragma__('noskip')


DIV = helpers.XmlConstructor.tagger("div")
FORM = helpers.XmlConstructor.tagger("form")
I = helpers.XmlConstructor.tagger("i")
CONCATENATE = helpers.CONCATENATE


__pragma__('kwargs')


class Modal():
    def __init__(self, target_selector, **parameters):
        self.target_selector = target_selector
        self.target_element = jQuery(target_selector)
        self.title = ""
        self.content = ""
        self.footer = ""
        self.buttons_panel = ""
        self.header_height = 80
        self.footer_height = 80
        self._max_content_height = 400
        self._form = parameters.get("form", None)
        _class_panel = ""
        self._close_on_click_outside = parameters.get("close_on_click_outside", True)
        if "title" in parameters:
            self.title = parameters["title"]
        if "content" in parameters:
            self.content = parameters["content"]
        if "footer" in parameters:
            self.footer = parameters["footer"]
        else:
            self.footer_height = 20
        if "buttons_panel" in parameters:
            _class_panel = " has_buttons_panel"
            self.buttons_panel = parameters["buttons_panel"]
        if "header_height" in parameters:
            self.header_height = parameters["header_height"]
        if "footer_height" in parameters:
            self.footer_height = parameters["footer_height"]
        if "_class" in parameters:
            parameters["_class"] = "{0}{1}{2}".format(
                parameters["_class"].strip(),
                " phanterpwa-component-modal-wrapper phanterpwa-container",
                _class_panel
            )
        else:
            parameters["_class"] = "phanterpwa-component-modal-wrapper phanterpwa-container{0}".format(_class_panel)
        if "after_open" in parameters:
            self._after_open = parameters["after_open"]

        self._on_close = parameters.get("on_close", None)
        self._after_close = parameters.get("after_close", None)
        self._z_index = parameters.get("z_index", 1006)
        style_close_button = None
        if self.header_height < 50:
            style_close_button = "line-height: {0}px; height: {0}px; font-size: 1rem;".format(self.header_height)
        wrapper_content = CONCATENATE(
            DIV(
                DIV(
                    I(_class="fas fa-times"),
                    _class="phanterpwa-component-modal-close link",
                    _style=style_close_button
                ),
                DIV(
                    snippets.Centralizer(
                        "phanterpwa-component-modal-title-snippets-centralizer",
                        DIV(self.title, _class="phanterpwa-component-modal-title-content"),
                        default_height=self.header_height - 20
                    ),
                    _class="phanterpwa-component-modal-title"
                ),
                _class="phanterpwa-component-modal-header-container"
            ),
            DIV(self.buttons_panel, _class="phanterpwa-component-modal-button-panel"),
            DIV(
                self.content,
                
                _class="phanterpwa-component-modal-content-container"
            ),
            DIV(
                self.footer,
                _class="phanterpwa-component-modal-footer-container"
            ),
        )
        if self._form is not None:
            if ["_class"] in parameters:
                parameters["_class"] = "{0}{1}".format(parameters["_class"], " phanterpwa-form")
            else:
                parameters["_class"] = "phanterpwa-form"
            parameters["_id"] = "form-{0}".format(self._form)
            parameters["_phanterpwa-form"] = self._form
            self.modal_wrapper = FORM(
                wrapper_content,
                **parameters
            )
        else:
            self.modal_wrapper = DIV(
                wrapper_content,
                **parameters
            )
        self.modal_container = DIV(
            DIV(
                DIV(
                    DIV(
                        DIV(
                            self.modal_wrapper,
                            _class="phanterpwa-centralizer-center"
                        ),
                        _class="phanterpwa-centralizer-horizontal"
                    ),
                    _class="phanterpwa-centralizer-vertical"
                ),
                _class="phanterpwa-centralizer-wrapper"
            ),
            _class="phanterpwa-component-modal-container phanterpwa-fixed-fulldisplay",
            _style="z-index: {0}".format(self._z_index)
        )

    def switch_modal(self):
        modal_container = jQuery(self.target_selector).find(".phanterpwa-component-modal-container")
        if modal_container.hasClass("enabled"):
            self.close()
        else:
            self.open()

    def close(self):
        if callable(self._on_close):
            self._on_close(self)
        modal_container = jQuery(self.target_selector).find(".phanterpwa-component-modal-container")
        modal_container.addClass("closing")
        setTimeout(
            lambda: modal_container.removeClass("enabled"),
            500
        )
        if callable(self._after_close):
            self._after_close(self)

    def _close_on_click_container(self, event, target_element):
        if self._close_on_click_outside is True:
            if (event.target is target_element):
                self.close()

    def open(self):
        self.start()
        modal_container = jQuery(self.target_selector).find(".phanterpwa-component-modal-container")
        modal_container.addClass("enabled")
        if callable(self._after_open):
            self._after_open(modal_container)

        def change_position_and_zindex(el):
            widget_name = jQuery(el).attr("phanterpwa-widget")
            widget = window.PhanterPWA.Request.widgets.get(widget_name, None)
            if widget is not None:
                if callable(widget.set_z_index):
                    widget.set_z_index(self._z_index + 1)
                if callable(widget.set_recalc_on_scroll):
                    widget.set_recalc_on_scroll(True)

        modal_container.find(
            "[phanterpwa-widget]"
        ).each(
            lambda: change_position_and_zindex(this)
        )

    def _calc_content_height(self):
        h = jQuery(window).height()
        self._max_content_height = h - self.header_height - self.footer_height - 20
        if h <= 0:
            h = 0
        self.target_element = jQuery(self.target_selector)
        self.target_element.find(
            ".phanterpwa-component-modal-header-container"
        ).css(
            "height",
             self.header_height
        )
        self.target_element.find(
            ".phanterpwa-component-modal-wrapper"
        ).css(
            "padding-bottom",
            self.footer_height
        ).css(
            "padding-top",
            self.header_height

        )
        self.target_element.find(
            ".phanterpwa-component-modal-footer-container"
        ).css(
            "height",
            self.footer_height
        )
        self.target_element.find(
            ".phanterpwa-component-modal-content-container"
        ).css(
            "max-height",
            self._max_content_height
        )

    def start(self):
        self.target_element = jQuery(self.target_selector)
        self.target_element.find('.phanterpwa-component-modal-container.phanterpwa-fixed-fulldisplay').remove()
        self.modal_container.append_to(self.target_selector)
        self._calc_content_height()
        jQuery(window).resize(lambda: self._calc_content_height())
        self.target_element.find(".phanterpwa-centralizer-horizontal").off(
            "click.phanterpwa-component-modal-container"
        ).on(
            "click.phanterpwa-component-modal-container",
            lambda e: self._close_on_click_container(e, this)
        )
        self.target_element.find(".phanterpwa-centralizer-center").off(
            "click.phanterpwa-component-modal-container"
        ).on(
            "click.phanterpwa-component-modal-container",
            lambda e: self._close_on_click_container(e, this)
        )
        self.target_element.find(".phanterpwa-component-modal-close").off(
            "click.phanterpwa-component-modal-container"
        ).on(
            "click.phanterpwa-component-modal-container",
            self.close
        )


__pragma__('nokwargs')
